from dataclasses import dataclass
from astrbot.core.config.astrbot_config import AstrBotConfig
from astrbot.core.star import PluginManager


@dataclass
class PipelineContext:
    """上下文对象，包含管道执行所需的上下文信息"""

    astrbot_config: AstrBotConfig  # AstrBot 配置对象
    plugin_manager: PluginManager  # 插件管理器对象



import uuid
import json
import os
from .route import Route, Response, RouteContext
from astrbot.core import web_chat_queue, web_chat_back_queue
from quart import request, Response as QuartResponse, g, make_response
from astrbot.core.db import BaseDatabase
import asyncio
from astrbot.core import logger
from astrbot.core.core_lifecycle import AstrBotCoreLifecycle
from astrbot.core.utils.astrbot_path import get_astrbot_data_path


class ChatRoute(Route):
    def __init__(
        self,
        context: RouteContext,
        db: BaseDatabase,
        core_lifecycle: AstrBotCoreLifecycle,
    ) -> None:
        super().__init__(context)
        self.routes = {
            "/chat/send": ("POST", self.chat),
            "/chat/listen": ("GET", self.listener),
            "/chat/new_conversation": ("GET", self.new_conversation),
            "/chat/conversations": ("GET", self.get_conversations),
            "/chat/get_conversation": ("GET", self.get_conversation),
            "/chat/delete_conversation": ("GET", self.delete_conversation),
            "/chat/get_file": ("GET", self.get_file),
            "/chat/post_image": ("POST", self.post_image),
            "/chat/post_file": ("POST", self.post_file),
            "/chat/status": ("GET", self.status),
        }
        self.db = db
        self.core_lifecycle = core_lifecycle
        self.register_routes()
        self.imgs_dir = os.path.join(get_astrbot_data_path(), "webchat", "imgs")
        os.makedirs(self.imgs_dir, exist_ok=True)

        self.supported_imgs = ["jpg", "jpeg", "png", "gif", "webp"]

        self.curr_user_cid = {}
        self.curr_chat_sse = {}

    async def status(self):
        has_llm_enabled = (
            self.core_lifecycle.provider_manager.curr_provider_inst is not None
        )
        has_stt_enabled = (
            self.core_lifecycle.provider_manager.curr_stt_provider_inst is not None
        )
        return (
            Response()
            .ok(data={"llm_enabled": has_llm_enabled, "stt_enabled": has_stt_enabled})
            .__dict__
        )

    async def get_file(self):
        filename = request.args.get("filename")
        if not filename:
            return Response().error("Missing key: filename").__dict__

        try:
            file_path = os.path.join(self.imgs_dir, os.path.basename(filename))
            real_file_path = os.path.realpath(file_path)
            real_imgs_dir = os.path.realpath(self.imgs_dir)

            if not real_file_path.startswith(real_imgs_dir):
                return Response().error("Invalid file path").__dict__

            with open(real_file_path, "rb") as f:
                filename_ext = os.path.splitext(filename)[1].lower()

                if filename_ext == ".wav":
                    return QuartResponse(f.read(), mimetype="audio/wav")
                elif filename_ext[1:] in self.supported_imgs:
                    return QuartResponse(f.read(), mimetype="image/jpeg")
                else:
                    return QuartResponse(f.read())

        except (FileNotFoundError, OSError):
            return Response().error("File access error").__dict__

    async def post_image(self):
        post_data = await request.files
        if "file" not in post_data:
            return Response().error("Missing key: file").__dict__

        file = post_data["file"]
        filename = str(uuid.uuid4()) + ".jpg"
        path = os.path.join(self.imgs_dir, filename)
        await file.save(path)

        return Response().ok(data={"filename": filename}).__dict__

    async def post_file(self):
        post_data = await request.files
        if "file" not in post_data:
            return Response().error("Missing key: file").__dict__

        file = post_data["file"]
        filename = f"{str(uuid.uuid4())}"
        print(file)
        # 通过文件格式判断文件类型
        if file.content_type.startswith("audio"):
            filename += ".wav"

        path = os.path.join(self.imgs_dir, filename)
        await file.save(path)

        return Response().ok(data={"filename": filename}).__dict__

    async def chat(self):
        username = g.get("username", "guest")

        post_data = await request.json
        if "message" not in post_data and "image_url" not in post_data:
            return Response().error("Missing key: message or image_url").__dict__

        if "conversation_id" not in post_data:
            return Response().error("Missing key: conversation_id").__dict__

        message = post_data["message"]
        conversation_id = post_data["conversation_id"]
        image_url = post_data.get("image_url")
        audio_url = post_data.get("audio_url")
        if not message and not image_url and not audio_url:
            return (
                Response()
                .error("Message and image_url and audio_url are empty")
                .__dict__
            )
        if not conversation_id:
            return Response().error("conversation_id is empty").__dict__

        self.curr_user_cid[username] = conversation_id

        await web_chat_queue.put((
            username,
            conversation_id,
            {
                "message": message,
                "image_url": image_url,  # list
                "audio_url": audio_url,
            },
        ))

        # 持久化
        conversation = self.db.get_conversation_by_user_id(username, conversation_id)
        try:
            history = json.loads(conversation.history)
        except BaseException as e:
            print(e)
            history = []
        new_his = {"type": "user", "message": message}
        if image_url:
            new_his["image_url"] = image_url
        if audio_url:
            new_his["audio_url"] = audio_url
        history.append(new_his)
        self.db.update_conversation(
            username, conversation_id, history=json.dumps(history)
        )

        return Response().ok().__dict__

    async def listener(self):
        """一直保持长连接"""

        username = g.get("username", "guest")

        if username in self.curr_chat_sse:
            return Response().error("Already connected").__dict__

        self.curr_chat_sse[username] = None

        heartbeat = json.dumps({"type": "heartbeat", "data": "ping"})

        async def stream():
            try:
                yield f"data: {heartbeat}\n\n"  # 心跳包
                while True:
                    try:
                        result = await asyncio.wait_for(
                            web_chat_back_queue.get(), timeout=10
                        )  # 设置超时时间为5秒
                    except asyncio.TimeoutError:
                        yield f"data: {heartbeat}\n\n"  # 心跳包
                        continue

                    if not result:
                        continue

                    result_text = result["data"]
                    type = result.get("type")
                    cid = result.get("cid")
                    streaming = result.get("streaming", False)
                    if cid != self.curr_user_cid.get(username):
                        # 丢弃
                        continue
                    yield f"data: {json.dumps(result, ensure_ascii=False)}\n\n"
                    await asyncio.sleep(0.05)

                    if streaming and type != "end":
                        continue

                    if result_text:
                        conversation = self.db.get_conversation_by_user_id(
                            username, cid
                        )
                        try:
                            history = json.loads(conversation.history)
                        except BaseException as e:
                            print(e)
                            history = []
                        history.append({"type": "bot", "message": result_text})
                        self.db.update_conversation(
                            username, cid, history=json.dumps(history)
                        )
            except BaseException as _:
                logger.debug(f"用户 {username} 断开聊天长连接。")
                self.curr_chat_sse.pop(username)
                return

        response = await make_response(
            stream(),
            {
                "Content-Type": "text/event-stream",
                "Cache-Control": "no-cache",
                "Transfer-Encoding": "chunked",
                "Connection": "keep-alive",
            },
        )
        response.timeout = None
        return response

    async def delete_conversation(self):
        username = g.get("username", "guest")
        conversation_id = request.args.get("conversation_id")
        if not conversation_id:
            return Response().error("Missing key: conversation_id").__dict__

        self.db.delete_conversation(username, conversation_id)
        return Response().ok().__dict__

    async def new_conversation(self):
        username = g.get("username", "guest")
        conversation_id = str(uuid.uuid4())
        self.db.new_conversation(username, conversation_id)
        return Response().ok(data={"conversation_id": conversation_id}).__dict__

    async def get_conversations(self):
        username = g.get("username", "guest")
        conversations = self.db.get_conversations(username)
        return Response().ok(data=conversations).__dict__

    async def get_conversation(self):
        username = g.get("username", "guest")
        conversation_id = request.args.get("conversation_id")
        if not conversation_id:
            return Response().error("Missing key: conversation_id").__dict__

        conversation = self.db.get_conversation_by_user_id(username, conversation_id)

        self.curr_user_cid[username] = conversation_id

        return Response().ok(data=conversation).__dict__



from .star import StarMetadata
from .star_manager import PluginManager
from .context import Context
from astrbot.core.provider import Provider
from astrbot.core.utils.command_parser import CommandParserMixin
from astrbot.core import html_renderer
from astrbot.core.star.star_tools import StarTools


class Star(CommandParserMixin):
    """所有插件（Star）的父类，所有插件都应该继承于这个类"""

    def __init__(self, context: Context):
        StarTools.initialize(context)
        self.context = context

    async def text_to_image(self, text: str, return_url=True) -> str:
        """将文本转换为图片"""
        return await html_renderer.render_t2i(text, return_url=return_url)

    async def html_render(self, tmpl: str, data: dict, return_url=True) -> str:
        """渲染 HTML"""
        return await html_renderer.render_custom_template(
            tmpl, data, return_url=return_url
        )

    async def terminate(self):
        """当插件被禁用、重载插件时会调用这个方法"""
        pass


__all__ = ["Star", "StarMetadata", "PluginManager", "Context", "Provider", "StarTools"]



import enum
from . import HandlerFilter
from astrbot.core.platform.astr_message_event import AstrMessageEvent
from astrbot.core.config import AstrBotConfig


class PermissionType(enum.Flag):
    """权限类型。当选择 MEMBER，ADMIN 也可以通过。"""

    ADMIN = enum.auto()
    MEMBER = enum.auto()


class PermissionTypeFilter(HandlerFilter):
    def __init__(self, permission_type: PermissionType, raise_error: bool = True):
        self.permission_type = permission_type
        self.raise_error = raise_error

    def filter(self, event: AstrMessageEvent, cfg: AstrBotConfig) -> bool:
        """过滤器"""
        if self.permission_type == PermissionType.ADMIN:
            if not event.is_admin():
                # event.stop_event()
                # raise ValueError(f"您 (ID: {event.get_sender_id()}) 没有权限操作管理员指令。")
                return False

        return True



from astrbot.core.config.astrbot_config import AstrBotConfig
from astrbot import logger
from astrbot.core import html_renderer
from astrbot.core import sp
from astrbot.core.star.register import register_llm_tool as llm_tool

__all__ = ["AstrBotConfig", "logger", "html_renderer", "llm_tool", "sp"]



from astrbot.core.message.message_event_result import (
    MessageEventResult,
    EventResultType,
)

from .waking_check.stage import WakingCheckStage
from .whitelist_check.stage import WhitelistCheckStage
from .rate_limit_check.stage import RateLimitStage
from .content_safety_check.stage import ContentSafetyCheckStage
from .platform_compatibility.stage import PlatformCompatibilityStage
from .preprocess_stage.stage import PreProcessStage
from .process_stage.stage import ProcessStage
from .result_decorate.stage import ResultDecorateStage
from .respond.stage import RespondStage

# 管道阶段顺序
STAGES_ORDER = [
    "WakingCheckStage",  # 检查是否需要唤醒
    "WhitelistCheckStage",  # 检查是否在群聊/私聊白名单
    "RateLimitStage",  # 检查会话是否超过频率限制
    "ContentSafetyCheckStage",  # 检查内容安全
    "PlatformCompatibilityStage",  # 检查所有处理器的平台兼容性
    "PreProcessStage",  # 预处理
    "ProcessStage",  # 交由 Stars 处理（a.k.a 插件），或者 LLM 调用
    "ResultDecorateStage",  # 处理结果，比如添加回复前缀、t2i、转换为语音 等
    "RespondStage",  # 发送消息
]

__all__ = [
    "WakingCheckStage",
    "WhitelistCheckStage",
    "RateLimitStage",
    "ContentSafetyCheckStage",
    "PlatformCompatibilityStage",
    "PreProcessStage",
    "ProcessStage",
    "ResultDecorateStage",
    "RespondStage",
    "MessageEventResult",
    "EventResultType",
]



from __future__ import annotations

from types import ModuleType
from typing import List, Dict
from dataclasses import dataclass, field
from astrbot.core.config import AstrBotConfig

star_registry: List[StarMetadata] = []
star_map: Dict[str, StarMetadata] = {}
"""key 是模块路径，__module__"""


@dataclass
class StarMetadata:
    """
    插件的元数据。

    当 activated 为 False 时，star_cls 可能为 None，请不要在插件未激活时调用 star_cls 的方法。
    """

    name: str
    author: str  # 插件作者
    desc: str  # 插件简介
    version: str  # 插件版本
    repo: str = None  # 插件仓库地址

    star_cls_type: type = None
    """插件的类对象的类型"""
    module_path: str = None
    """插件的模块路径"""

    star_cls: object = None
    """插件的类对象"""
    module: ModuleType = None
    """插件的模块对象"""
    root_dir_name: str = None
    """插件的目录名称"""
    reserved: bool = False
    """是否是 AstrBot 的保留插件"""

    activated: bool = True
    """是否被激活"""

    config: AstrBotConfig = None
    """插件配置"""

    star_handler_full_names: List[str] = field(default_factory=list)
    """注册的 Handler 的全名列表"""

    supported_platforms: Dict[str, bool] = field(default_factory=dict)
    """插件支持的平台ID字典，key为平台ID，value为是否支持"""

    def __str__(self) -> str:
        return f"StarMetadata({self.name}, {self.desc}, {self.version}, {self.repo})"

    def update_platform_compatibility(self, plugin_enable_config: dict) -> None:
        """更新插件支持的平台列表

        Args:
            plugin_enable_config: 平台插件启用配置，即platform_settings.plugin_enable配置项
        """
        if not plugin_enable_config:
            return

        # 清空之前的配置
        self.supported_platforms.clear()

        # 遍历所有平台配置
        for platform_id, plugins in plugin_enable_config.items():
            # 检查该插件在当前平台的配置
            if self.name in plugins:
                self.supported_platforms[platform_id] = plugins[self.name]
            else:
                # 如果没有明确配置，默认为启用
                self.supported_platforms[platform_id] = True



"""
会话控制
"""

import abc
import asyncio
import time
import functools
import copy
import astrbot.core.message.components as Comp
from typing import Dict, Any, Callable, Awaitable, List
from astrbot.core.platform import AstrMessageEvent

USER_SESSIONS: Dict[str, "SessionWaiter"] = {}  # 存储 SessionWaiter 实例
FILTERS: List["SessionFilter"] = []  # 存储 SessionFilter 实例


class SessionController:
    """
    控制一个 Session 是否已经结束
    """

    def __init__(self):
        self.future = asyncio.Future()
        self.current_event: asyncio.Event = None
        """当前正在等待的所用的异步事件"""
        self.ts: float = None
        """上次保持(keep)开始时的时间"""
        self.timeout: float | int = None
        """上次保持(keep)开始时的超时时间"""

        self.history_chains: List[List[Comp.BaseMessageComponent]] = []

    def stop(self, error: Exception = None):
        """立即结束这个会话"""
        if not self.future.done():
            if error:
                self.future.set_exception(error)
            else:
                self.future.set_result(None)

    def keep(self, timeout: float | int = 0, reset_timeout=False):
        """保持这个会话

        Args:
            timeout (float): 必填。会话超时时间。
            当 reset_timeout 设置为 True 时, 代表重置超时时间, timeout 必须 > 0, 如果 <= 0 则立即结束会话。
            当 reset_timeout 设置为 False 时, 代表继续维持原来的超时时间, 新 timeout = 原来剩余的timeout + timeout (可以 < 0)
        """
        new_ts = time.time()

        if reset_timeout:
            if timeout <= 0:
                self.stop()
                return
        else:
            left_timeout = self.timeout - (new_ts - self.ts)
            timeout = left_timeout + timeout
            if timeout <= 0:
                self.stop()
                return

        if self.current_event and not self.current_event.is_set():
            self.current_event.set()  # 通知上一个 keep 结束

        new_event = asyncio.Event()
        self.ts = new_ts
        self.current_event = new_event
        self.timeout = timeout

        asyncio.create_task(self._holding(new_event, timeout))  # 开始新的 keep

    async def _holding(self, event: asyncio.Event, timeout: int):
        """等待事件结束或超时"""
        try:
            await asyncio.wait_for(event.wait(), timeout)
        except asyncio.TimeoutError:
            if not self.future.done():
                self.future.set_exception(TimeoutError("等待超时"))
        except asyncio.CancelledError:
            pass  # 避免报错
        # finally:

    def get_history_chains(self) -> List[List[Comp.BaseMessageComponent]]:
        """获取历史消息链"""
        return self.history_chains


class SessionFilter:
    """如何界定一个会话"""

    @abc.abstractmethod
    def filter(self, event: AstrMessageEvent) -> str:
        """根据事件返回一个会话标识符"""
        pass


class DefaultSessionFilter(SessionFilter):
    def filter(self, event: AstrMessageEvent) -> str:
        """默认实现，返回统一消息来源字符串作为会话标识符"""
        return event.unified_msg_origin


class SessionWaiter:
    def __init__(
        self,
        session_filter: SessionFilter,
        session_id: str,
        record_history_chains: bool,
    ):
        self.session_id = session_id
        self.session_filter = session_filter
        self.handler: Callable[[str], Awaitable[Any]] | None = None  # 处理函数

        self.session_controller = SessionController()
        self.record_history_chains = record_history_chains
        """是否记录历史消息链"""

        self._lock = asyncio.Lock()
        """需要保证一个 session 同时只有一个 trigger"""

    async def register_wait(
        self, handler: Callable[[str], Awaitable[Any]], timeout: int = 30
    ) -> Any:
        """等待外部输入并处理"""
        self.handler = handler
        USER_SESSIONS[self.session_id] = self

        # 开始一个会话保持事件
        self.session_controller.keep(timeout, reset_timeout=True)

        try:
            return await self.session_controller.future
        except Exception as e:
            self._cleanup(e)
            raise e
        finally:
            self._cleanup()

    def _cleanup(self, error: Exception = None):
        """清理会话"""
        USER_SESSIONS.pop(self.session_id, None)
        try:
            FILTERS.remove(self.session_filter)
        except ValueError:
            pass
        self.session_controller.stop(error)

    @classmethod
    async def trigger(cls, session_id: str, event: AstrMessageEvent):
        """外部输入触发会话处理"""
        session = USER_SESSIONS.get(session_id, None)
        if not session or session.session_controller.future.done():
            return

        async with session._lock:
            if not session.session_controller.future.done():
                if session.record_history_chains:
                    session.session_controller.history_chains.append(
                        [copy.deepcopy(comp) for comp in event.get_messages()]
                    )
                try:
                    # TODO: 这里使用 create_task，跟踪 task，防止超时后这里 handler 仍然在执行
                    await session.handler(session.session_controller, event)
                except Exception as e:
                    session.session_controller.stop(e)


def session_waiter(timeout: int = 30, record_history_chains: bool = False):
    """
    装饰器：自动将函数注册为 SessionWaiter 处理函数，并等待外部输入触发执行。

    :param timeout: 超时时间（秒）
    :param record_history_chain: 是否自动记录历史消息链。可以通过 controller.get_history_chains() 获取。深拷贝。
    """

    def decorator(func: Callable[[str], Awaitable[Any]]):
        @functools.wraps(func)
        async def wrapper(
            event: AstrMessageEvent,
            session_filter: SessionFilter = None,
            *args,
            **kwargs,
        ):
            if not session_filter:
                session_filter = DefaultSessionFilter()
            if not isinstance(session_filter, SessionFilter):
                raise ValueError("session_filter 必须是 SessionFilter")

            session_id = session_filter.filter(event)
            FILTERS.append(session_filter)

            waiter = SessionWaiter(session_filter, session_id, record_history_chains)
            return await waiter.register_wait(func, timeout)

        return wrapper

    return decorator



import json
import os
import traceback

import aiohttp
from quart import request

from astrbot.core import logger
from astrbot.core.core_lifecycle import AstrBotCoreLifecycle
from astrbot.core.utils.astrbot_path import get_astrbot_data_path

from .route import Response, Route, RouteContext

DEFAULT_MCP_CONFIG = {"mcpServers": {}}


class ToolsRoute(Route):
    def __init__(
        self, context: RouteContext, core_lifecycle: AstrBotCoreLifecycle
    ) -> None:
        super().__init__(context)
        self.core_lifecycle = core_lifecycle
        self.routes = {
            "/tools/mcp/servers": ("GET", self.get_mcp_servers),
            "/tools/mcp/add": ("POST", self.add_mcp_server),
            "/tools/mcp/update": ("POST", self.update_mcp_server),
            "/tools/mcp/delete": ("POST", self.delete_mcp_server),
            "/tools/mcp/market": ("GET", self.get_mcp_markets),
        }
        self.register_routes()
        self.tool_mgr = self.core_lifecycle.provider_manager.llm_tools

    @property
    def mcp_config_path(self):
        data_dir = get_astrbot_data_path()
        return os.path.join(data_dir, "mcp_server.json")

    def load_mcp_config(self):
        if not os.path.exists(self.mcp_config_path):
            # 配置文件不存在，创建默认配置
            os.makedirs(os.path.dirname(self.mcp_config_path), exist_ok=True)
            with open(self.mcp_config_path, "w", encoding="utf-8") as f:
                json.dump(DEFAULT_MCP_CONFIG, f, ensure_ascii=False, indent=4)
            return DEFAULT_MCP_CONFIG

        try:
            with open(self.mcp_config_path, "r", encoding="utf-8") as f:
                return json.load(f)
        except Exception as e:
            logger.error(f"加载 MCP 配置失败: {e}")
            return DEFAULT_MCP_CONFIG

    def save_mcp_config(self, config):
        try:
            with open(self.mcp_config_path, "w", encoding="utf-8") as f:
                json.dump(config, f, ensure_ascii=False, indent=4)
            return True
        except Exception as e:
            logger.error(f"保存 MCP 配置失败: {e}")
            return False

    async def get_mcp_servers(self):
        try:
            config = self.load_mcp_config()
            servers = []

            # 获取所有服务器并添加它们的工具列表
            for name, server_config in config["mcpServers"].items():
                server_info = {
                    "name": name,
                    "active": server_config.get("active", True),
                }

                # 复制所有配置字段
                for key, value in server_config.items():
                    if key != "active":  # active 已经处理
                        server_info[key] = value

                # 如果MCP客户端已初始化，从客户端获取工具名称
                for (
                    name_key,
                    mcp_client,
                ) in self.tool_mgr.mcp_client_dict.items():
                    if name_key == name:
                        server_info["tools"] = [tool.name for tool in mcp_client.tools]
                        server_info["errlogs"] = mcp_client.server_errlogs
                        break
                else:
                    server_info["tools"] = []

                servers.append(server_info)

            return Response().ok(servers).__dict__
        except Exception as e:
            logger.error(traceback.format_exc())
            return Response().error(f"获取 MCP 服务器列表失败: {str(e)}").__dict__

    async def add_mcp_server(self):
        try:
            server_data = await request.json

            name = server_data.get("name", "")

            # 检查必填字段
            if not name:
                return Response().error("服务器名称不能为空").__dict__

            # 移除特殊字段并检查配置是否有效
            has_valid_config = False
            server_config = {"active": server_data.get("active", True)}

            # 复制所有配置字段
            for key, value in server_data.items():
                if key not in ["name", "active", "tools", "errlogs"]:  # 排除特殊字段
                    if key == "mcpServers":
                        key_0 = list(server_data["mcpServers"].keys())[
                            0
                        ]  # 不考虑为空的情况
                        server_config = server_data["mcpServers"][key_0]
                    else:
                        server_config[key] = value
                    has_valid_config = True

            if not has_valid_config:
                return Response().error("必须提供有效的服务器配置").__dict__

            config = self.load_mcp_config()

            if name in config["mcpServers"]:
                return Response().error(f"服务器 {name} 已存在").__dict__

            config["mcpServers"][name] = server_config

            if self.save_mcp_config(config):
                # 动态初始化新MCP客户端
                await self.tool_mgr.mcp_service_queue.put({
                    "type": "init",
                    "name": name,
                    "cfg": config["mcpServers"][name],
                })
                return Response().ok(None, f"成功添加 MCP 服务器 {name}").__dict__
            else:
                return Response().error("保存配置失败").__dict__
        except Exception as e:
            logger.error(traceback.format_exc())
            return Response().error(f"添加 MCP 服务器失败: {str(e)}").__dict__

    async def update_mcp_server(self):
        try:
            server_data = await request.json

            name = server_data.get("name", "")

            if not name:
                return Response().error("服务器名称不能为空").__dict__

            config = self.load_mcp_config()

            if name not in config["mcpServers"]:
                return Response().error(f"服务器 {name} 不存在").__dict__

            # 获取活动状态
            active = server_data.get(
                "active", config["mcpServers"][name].get("active", True)
            )

            # 创建新的配置对象
            server_config = {"active": active}

            # 仅更新活动状态的特殊处理
            only_update_active = True

            # 复制所有配置字段
            for key, value in server_data.items():
                if key not in ["name", "active", "tools", "errlogs"]:  # 排除特殊字段
                    if key == "mcpServers":
                        key_0 = list(server_data["mcpServers"].keys())[
                            0
                        ]  # 不考虑为空的情况
                        server_config = server_data["mcpServers"][key_0]
                    else:
                        server_config[key] = value
                    only_update_active = False

            # 如果只更新活动状态，保留原始配置
            if only_update_active:
                for key, value in config["mcpServers"][name].items():
                    if key != "active":  # 除了active之外的所有字段都保留
                        server_config[key] = value

            config["mcpServers"][name] = server_config

            if self.save_mcp_config(config):
                # 处理MCP客户端状态变化
                if active:
                    # 如果要激活服务器或者配置已更改
                    if name in self.tool_mgr.mcp_client_dict or not only_update_active:
                        await self.tool_mgr.mcp_service_queue.put({
                            "type": "terminate",
                            "name": name,
                        })
                        await self.tool_mgr.mcp_service_queue.put({
                            "type": "init",
                            "name": name,
                            "cfg": config["mcpServers"][name],
                        })
                    else:
                        # 客户端不存在，初始化
                        await self.tool_mgr.mcp_service_queue.put({
                            "type": "init",
                            "name": name,
                            "cfg": config["mcpServers"][name],
                        })
                else:
                    # 如果要停用服务器
                    if name in self.tool_mgr.mcp_client_dict:
                        self.tool_mgr.mcp_service_queue.put_nowait({
                            "type": "terminate",
                            "name": name,
                        })

                return Response().ok(None, f"成功更新 MCP 服务器 {name}").__dict__
            else:
                return Response().error("保存配置失败").__dict__
        except Exception as e:
            logger.error(traceback.format_exc())
            return Response().error(f"更新 MCP 服务器失败: {str(e)}").__dict__

    async def delete_mcp_server(self):
        try:
            server_data = await request.json
            name = server_data.get("name", "")

            if not name:
                return Response().error("服务器名称不能为空").__dict__

            config = self.load_mcp_config()

            if name not in config["mcpServers"]:
                return Response().error(f"服务器 {name} 不存在").__dict__

            # 删除服务器配置
            del config["mcpServers"][name]

            if self.save_mcp_config(config):
                # 关闭并删除MCP客户端
                if name in self.tool_mgr.mcp_client_dict:
                    self.tool_mgr.mcp_service_queue.put_nowait({
                        "type": "terminate",
                        "name": name,
                    })

                return Response().ok(None, f"成功删除 MCP 服务器 {name}").__dict__
            else:
                return Response().error("保存配置失败").__dict__
        except Exception as e:
            logger.error(traceback.format_exc())
            return Response().error(f"删除 MCP 服务器失败: {str(e)}").__dict__

    async def get_mcp_markets(self):
        page = request.args.get("page", 1, type=int)
        page_size = request.args.get("page_size", 10, type=int)
        BASE_URL = (
            "https://api.soulter.top/astrbot/mcpservers?page={}&page_size={}".format(
                page,
                page_size,
            )
        )
        try:
            async with aiohttp.ClientSession() as session:
                async with session.get(f"{BASE_URL}") as response:
                    if response.status == 200:
                        data = await response.json()
                        return Response().ok(data["data"]).__dict__
                    else:
                        return (
                            Response()
                            .error(f"获取市场数据失败: HTTP {response.status}")
                            .__dict__
                        )
        except Exception as _:
            logger.error(traceback.format_exc())
        return Response().error("获取市场数据失败").__dict__



from ..star import star_registry, StarMetadata, star_map


def register_star(name: str, author: str, desc: str, version: str, repo: str = None):
    """注册一个插件(Star)。

    Args:
        name: 插件名称。
        author: 作者。
        desc: 插件的简述。
        version: 版本号。
        repo: 仓库地址。如果没有填写仓库地址，将无法更新这个插件。

    如果需要为插件填写帮助信息，请使用如下格式：

    ```python
    class MyPlugin(star.Star):
        \'\'\'这是帮助信息\'\'\'
        ...

    帮助信息会被自动提取。使用 `/plugin <插件名> 可以查看帮助信息。`
    """

    def decorator(cls):
        star_metadata = StarMetadata(
            name=name,
            author=author,
            desc=desc,
            version=version,
            repo=repo,
            star_cls_type=cls,
            module_path=cls.__module__,
        )
        star_registry.append(star_metadata)
        star_map[cls.__module__] = star_metadata
        return cls

    return decorator



"""
日志系统, 用于支持核心组件和插件的日志记录, 提供了日志订阅功能

const:
    CACHED_SIZE: 日志缓存大小, 用于限制缓存的日志数量
    log_color_config: 日志颜色配置, 定义了不同日志级别的颜色

class:
    LogBroker: 日志代理类, 用于缓存和分发日志消息
    LogQueueHandler: 日志处理器, 用于将日志消息发送到 LogBroker
    LogManager: 日志管理器, 用于创建和配置日志记录器

function:
    is_plugin_path: 检查文件路径是否来自插件目录
    get_short_level_name: 将日志级别名称转换为四个字母的缩写

工作流程:
1. 通过 LogManager.GetLogger() 获取日志器, 配置了控制台输出和多个格式化过滤器
2. 通过 set_queue_handler() 设置日志处理器, 将日志消息发送到 LogBroker
3. logBroker 维护一个订阅者列表, 负责将日志分发给所有订阅者
4. 订阅者可以使用 register() 方法注册到 LogBroker, 订阅日志流
"""

import logging
import colorlog
import asyncio
import os
import sys
from collections import deque
from asyncio import Queue
from typing import List

# 日志缓存大小
CACHED_SIZE = 200
# 日志颜色配置
log_color_config = {
    "DEBUG": "green",
    "INFO": "bold_cyan",
    "WARNING": "bold_yellow",
    "ERROR": "red",
    "CRITICAL": "bold_red",
    "RESET": "reset",
    "asctime": "green",
}


def is_plugin_path(pathname):
    """检查文件路径是否来自插件目录

    Args:
        pathname (str): 文件路径

    Returns:
        bool: 如果路径来自插件目录，则返回 True，否则返回 False
    """
    if not pathname:
        return False

    norm_path = os.path.normpath(pathname)
    return ("data/plugins" in norm_path) or ("packages/" in norm_path)


def get_short_level_name(level_name):
    """将日志级别名称转换为四个字母的缩写

    Args:
        level_name (str): 日志级别名称, 如 "DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"

    Returns:
        str: 四个字母的日志级别缩写
    """
    level_map = {
        "DEBUG": "DBUG",
        "INFO": "INFO",
        "WARNING": "WARN",
        "ERROR": "ERRO",
        "CRITICAL": "CRIT",
    }
    return level_map.get(level_name, level_name[:4].upper())


class LogBroker:
    """日志代理类, 用于缓存和分发日志消息

    发布-订阅模式
    """

    def __init__(self):
        self.log_cache = deque(maxlen=CACHED_SIZE)  # 环形缓冲区, 保存最近的日志
        self.subscribers: List[Queue] = []  # 订阅者列表

    def register(self) -> Queue:
        """注册新的订阅者, 并给每个订阅者返回一个带有日志缓存的队列

        Returns:
            Queue: 订阅者的队列, 可用于接收日志消息
        """
        q = Queue(maxsize=CACHED_SIZE + 10)
        for log in self.log_cache:
            q.put_nowait(log)
        self.subscribers.append(q)
        return q

    def unregister(self, q: Queue):
        """取消订阅

        Args:
            q (Queue): 需要取消订阅的队列
        """
        self.subscribers.remove(q)

    def publish(self, log_entry: dict):
        """发布新日志到所有订阅者, 使用非阻塞方式投递, 避免一个订阅者阻塞整个系统

        Args:
            log_entry (dict): 日志消息, 包含日志级别和日志内容.
                example: {"level": "INFO", "data": "This is a log message.", "time": "2023-10-01 12:00:00"}
        """
        self.log_cache.append(log_entry)
        for q in self.subscribers:
            try:
                q.put_nowait(log_entry)
            except asyncio.QueueFull:
                pass


class LogQueueHandler(logging.Handler):
    """日志处理器, 用于将日志消息发送到 LogBroker

    继承自 logging.Handler
    """

    def __init__(self, log_broker: LogBroker):
        super().__init__()
        self.log_broker = log_broker

    def emit(self, record):
        """日志处理的入口方法, 接受一个日志记录, 转换为字符串后由 LogBroker 发布
        这个方法会在每次日志记录时被调用

        Args:
            record (logging.LogRecord): 日志记录对象, 包含日志信息
        """
        log_entry = self.format(record)
        self.log_broker.publish(
            {
                "level": record.levelname,
                "time": record.asctime,
                "data": log_entry,
            }
        )


class LogManager:
    """日志管理器, 用于创建和配置日志记录器

    提供了获取默认日志记录器logger和设置队列处理器的方法
    """

    @classmethod
    def GetLogger(cls, log_name: str = "default"):
        """获取指定名称的日志记录器logger

        Args:
            log_name (str): 日志记录器的名称, 默认为 "default"

        Returns:
            logging.Logger: 返回配置好的日志记录器
        """
        logger = logging.getLogger(log_name)
        # 检查该logger或父级logger是否已经有处理器, 如果已经有处理器, 直接返回该logger, 避免重复配置
        if logger.hasHandlers():
            return logger
        # 如果logger没有处理器
        console_handler = logging.StreamHandler(
            sys.stdout
        )  # 创建一个StreamHandler用于控制台输出
        console_handler.setLevel(
            logging.DEBUG
        )  # 将日志级别设置为DEBUG(最低级别, 显示所有日志), *如果插件没有设置级别, 默认为DEBUG

        # 创建彩色日志格式化器, 输出日志格式为: [时间] [插件标签] [日志级别] [文件名:行号]: 日志消息
        console_formatter = colorlog.ColoredFormatter(
            fmt="%(log_color)s [%(asctime)s] %(plugin_tag)s [%(short_levelname)-4s] [%(filename)s:%(lineno)d]: %(message)s %(reset)s",
            datefmt="%H:%M:%S",
            log_colors=log_color_config,
        )

        class PluginFilter(logging.Filter):
            """插件过滤器类, 用于标记日志来源是插件还是核心组件"""

            def filter(self, record):
                record.plugin_tag = (
                    "[Plug]" if is_plugin_path(record.pathname) else "[Core]"
                )
                return True

        class FileNameFilter(logging.Filter):
            """文件名过滤器类, 用于修改日志记录的文件名格式
            例如: 将文件路径 /path/to/file.py 转换为 file.<file> 格式"""

            # 获取这个文件和父文件夹的名字：<folder>.<file> 并且去除 .py
            def filter(self, record):
                dirname = os.path.dirname(record.pathname)
                record.filename = (
                    os.path.basename(dirname)
                    + "."
                    + os.path.basename(record.pathname).replace(".py", "")
                )
                return True

        class LevelNameFilter(logging.Filter):
            """短日志级别名称过滤器类, 用于将日志级别名称转换为四个字母的缩写"""

            # 添加短日志级别名称
            def filter(self, record):
                record.short_levelname = get_short_level_name(record.levelname)
                return True

        console_handler.setFormatter(console_formatter)  # 设置处理器的格式化器
        logger.addFilter(PluginFilter())  # 添加插件过滤器
        logger.addFilter(FileNameFilter())  # 添加文件名过滤器
        logger.addFilter(LevelNameFilter())  # 添加级别名称过滤器
        logger.setLevel(logging.DEBUG)  # 设置日志级别为DEBUG
        logger.addHandler(console_handler)  # 添加处理器到logger

        return logger

    @classmethod
    def set_queue_handler(cls, logger: logging.Logger, log_broker: LogBroker):
        """设置队列处理器, 用于将日志消息发送到 LogBroker

        Args:
            logger (logging.Logger): 日志记录器
            log_broker (LogBroker): 日志代理类, 用于缓存和分发日志消息
        """
        handler = LogQueueHandler(log_broker)
        handler.setLevel(logging.DEBUG)
        if logger.handlers:
            handler.setFormatter(logger.handlers[0].formatter)
        else:
            # 为队列处理器设置相同格式的formatter
            handler.setFormatter(
                logging.Formatter(
                    "[%(asctime)s] [%(short_levelname)s] %(plugin_tag)s[%(filename)s:%(lineno)d]: %(message)s"
                )
            )
        logger.addHandler(handler)



"""
MIT License

Copyright (c) 2021 Lxns-Network

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

import asyncio
import base64
import json
import os
import typing as T
import uuid
from enum import Enum

from pydantic.v1 import BaseModel

from astrbot.core import astrbot_config, file_token_service, logger
from astrbot.core.utils.astrbot_path import get_astrbot_data_path
from astrbot.core.utils.io import download_file, download_image_by_url, file_to_base64


class ComponentType(Enum):
    Plain = "Plain"  # 纯文本消息
    Face = "Face"  # QQ表情
    Record = "Record"  # 语音
    Video = "Video"  # 视频
    At = "At"  # At
    Node = "Node"  # 转发消息的一个节点
    Nodes = "Nodes"  # 转发消息的多个节点
    Poke = "Poke"  # QQ 戳一戳
    Image = "Image"  # 图片
    Reply = "Reply"  # 回复
    Forward = "Forward"  # 转发消息
    File = "File"  # 文件

    RPS = "RPS"  # TODO
    Dice = "Dice"  # TODO
    Shake = "Shake"  # TODO
    Anonymous = "Anonymous"  # TODO
    Share = "Share"
    Contact = "Contact"  # TODO
    Location = "Location"  # TODO
    Music = "Music"
    RedBag = "RedBag"
    Xml = "Xml"
    Json = "Json"
    CardImage = "CardImage"
    TTS = "TTS"
    Unknown = "Unknown"

    WechatEmoji = "WechatEmoji"  # Wechat 下的 emoji 表情包


class BaseMessageComponent(BaseModel):
    type: ComponentType

    def toString(self):
        output = f"[CQ:{self.type.lower()}"
        for k, v in self.__dict__.items():
            if k == "type" or v is None:
                continue
            if k == "_type":
                k = "type"
            if isinstance(v, bool):
                v = 1 if v else 0
            output += ",%s=%s" % (
                k,
                str(v)
                .replace("&", "&amp;")
                .replace(",", "&#44;")
                .replace("[", "&#91;")
                .replace("]", "&#93;"),
            )
        output += "]"
        return output

    def toDict(self):
        data = {}
        for k, v in self.__dict__.items():
            if k == "type" or v is None:
                continue
            if k == "_type":
                k = "type"
            data[k] = v
        return {"type": self.type.lower(), "data": data}

    async def to_dict(self) -> dict:
        # 默认情况下，回退到旧的同步 toDict()
        return self.toDict()


class Plain(BaseMessageComponent):
    type: ComponentType = "Plain"
    text: str
    convert: T.Optional[bool] = True  # 若为 False 则直接发送未转换 CQ 码的消息

    def __init__(self, text: str, convert: bool = True, **_):
        super().__init__(text=text, convert=convert, **_)

    def toString(self):  # 没有 [CQ:plain] 这种东西，所以直接导出纯文本
        if not self.convert:
            return self.text
        return (
            self.text.replace("&", "&amp;").replace("[", "&#91;").replace("]", "&#93;")
        )

    def toDict(self):
        return {"type": "text", "data": {"text": self.text.strip()}}


class Face(BaseMessageComponent):
    type: ComponentType = "Face"
    id: int

    def __init__(self, **_):
        super().__init__(**_)


class Record(BaseMessageComponent):
    type: ComponentType = "Record"
    file: T.Optional[str] = ""
    magic: T.Optional[bool] = False
    url: T.Optional[str] = ""
    cache: T.Optional[bool] = True
    proxy: T.Optional[bool] = True
    timeout: T.Optional[int] = 0
    # 额外
    path: T.Optional[str]

    def __init__(self, file: T.Optional[str], **_):
        for k in _.keys():
            if k == "url":
                pass
                # Protocol.warn(f"go-cqhttp doesn't support send {self.type} by {k}")
        super().__init__(file=file, **_)

    @staticmethod
    def fromFileSystem(path, **_):
        return Record(file=f"file:///{os.path.abspath(path)}", path=path, **_)

    @staticmethod
    def fromURL(url: str, **_):
        if url.startswith("http://") or url.startswith("https://"):
            return Record(file=url, **_)
        raise Exception("not a valid url")

    async def convert_to_file_path(self) -> str:
        """将这个语音统一转换为本地文件路径。这个方法避免了手动判断语音数据类型，直接返回语音数据的本地路径（如果是网络 URL, 则会自动进行下载）。

        Returns:
            str: 语音的本地路径，以绝对路径表示。
        """
        if self.file and self.file.startswith("file:///"):
            file_path = self.file[8:]
            return file_path
        elif self.file and self.file.startswith("http"):
            file_path = await download_image_by_url(self.file)
            return os.path.abspath(file_path)
        elif self.file and self.file.startswith("base64://"):
            bs64_data = self.file.removeprefix("base64://")
            image_bytes = base64.b64decode(bs64_data)
            temp_dir = os.path.join(get_astrbot_data_path(), "temp")
            file_path = os.path.join(temp_dir, f"{uuid.uuid4()}.jpg")
            with open(file_path, "wb") as f:
                f.write(image_bytes)
            return os.path.abspath(file_path)
        elif os.path.exists(self.file):
            file_path = self.file
            return os.path.abspath(file_path)
        else:
            raise Exception(f"not a valid file: {self.file}")

    async def convert_to_base64(self) -> str:
        """将语音统一转换为 base64 编码。这个方法避免了手动判断语音数据类型，直接返回语音数据的 base64 编码。

        Returns:
            str: 语音的 base64 编码，不以 base64:// 或者 data:image/jpeg;base64, 开头。
        """
        # convert to base64
        if self.file and self.file.startswith("file:///"):
            bs64_data = file_to_base64(self.file[8:])
        elif self.file and self.file.startswith("http"):
            file_path = await download_image_by_url(self.file)
            bs64_data = file_to_base64(file_path)
        elif self.file and self.file.startswith("base64://"):
            bs64_data = self.file
        elif os.path.exists(self.file):
            bs64_data = file_to_base64(self.file)
        else:
            raise Exception(f"not a valid file: {self.file}")
        bs64_data = bs64_data.removeprefix("base64://")
        return bs64_data

    async def register_to_file_service(self) -> str:
        """
        将语音注册到文件服务。

        Returns:
            str: 注册后的URL

        Raises:
            Exception: 如果未配置 callback_api_base
        """
        callback_host = astrbot_config.get("callback_api_base")

        if not callback_host:
            raise Exception("未配置 callback_api_base，文件服务不可用")

        file_path = await self.convert_to_file_path()

        token = await file_token_service.register_file(file_path)

        logger.debug(f"已注册：{callback_host}/api/file/{token}")

        return f"{callback_host}/api/file/{token}"


class Video(BaseMessageComponent):
    type: ComponentType = "Video"
    file: str
    cover: T.Optional[str] = ""
    c: T.Optional[int] = 2
    # 额外
    path: T.Optional[str] = ""

    def __init__(self, file: str, **_):
        super().__init__(file=file, **_)

    @staticmethod
    def fromFileSystem(path, **_):
        return Video(file=f"file:///{os.path.abspath(path)}", path=path, **_)

    @staticmethod
    def fromURL(url: str, **_):
        if url.startswith("http://") or url.startswith("https://"):
            return Video(file=url, **_)
        raise Exception("not a valid url")

    async def convert_to_file_path(self) -> str:
        """将这个视频统一转换为本地文件路径。这个方法避免了手动判断视频数据类型，直接返回视频数据的本地路径（如果是网络 URL，则会自动进行下载）。

        Returns:
            str: 视频的本地路径，以绝对路径表示。
        """
        url = self.file
        if url and url.startswith("file:///"):
            return url[8:]
        elif url and url.startswith("http"):
            download_dir = os.path.join(get_astrbot_data_path(), "temp")
            video_file_path = os.path.join(download_dir, f"{uuid.uuid4().hex}")
            await download_file(url, video_file_path)
            if os.path.exists(video_file_path):
                return os.path.abspath(video_file_path)
            else:
                raise Exception(f"download failed: {url}")
        elif os.path.exists(url):
            return os.path.abspath(url)
        else:
            raise Exception(f"not a valid file: {url}")

    async def register_to_file_service(self):
        """
        将视频注册到文件服务。

        Returns:
            str: 注册后的URL

        Raises:
            Exception: 如果未配置 callback_api_base
        """
        callback_host = astrbot_config.get("callback_api_base")

        if not callback_host:
            raise Exception("未配置 callback_api_base，文件服务不可用")

        file_path = await self.convert_to_file_path()

        token = await file_token_service.register_file(file_path)

        logger.debug(f"已注册：{callback_host}/api/file/{token}")

        return f"{callback_host}/api/file/{token}"

    async def to_dict(self):
        """需要和 toDict 区分开，toDict 是同步方法"""
        url_or_path = self.file
        if url_or_path.startswith("http"):
            payload_file = url_or_path
        elif callback_host := astrbot_config.get("callback_api_base"):
            callback_host = str(callback_host).removesuffix("/")
            token = await file_token_service.register_file(url_or_path)
            payload_file = f"{callback_host}/api/file/{token}"
            logger.debug(f"Generated video file callback link: {payload_file}")
        else:
            payload_file = url_or_path
        return {
            "type": "video",
            "data": {
                "file": payload_file,
            },
        }


class At(BaseMessageComponent):
    type: ComponentType = "At"
    qq: T.Union[int, str]  # 此处str为all时代表所有人
    name: T.Optional[str] = ""

    def __init__(self, **_):
        super().__init__(**_)

    def toDict(self):
        return {
            "type": "at",
            "data": {"qq": str(self.qq)},
        }


class AtAll(At):
    qq: str = "all"

    def __init__(self, **_):
        super().__init__(**_)


class RPS(BaseMessageComponent):  # TODO
    type: ComponentType = "RPS"

    def __init__(self, **_):
        super().__init__(**_)


class Dice(BaseMessageComponent):  # TODO
    type: ComponentType = "Dice"

    def __init__(self, **_):
        super().__init__(**_)


class Shake(BaseMessageComponent):  # TODO
    type: ComponentType = "Shake"

    def __init__(self, **_):
        super().__init__(**_)


class Anonymous(BaseMessageComponent):  # TODO
    type: ComponentType = "Anonymous"
    ignore: T.Optional[bool] = False

    def __init__(self, **_):
        super().__init__(**_)


class Share(BaseMessageComponent):
    type: ComponentType = "Share"
    url: str
    title: str
    content: T.Optional[str] = ""
    image: T.Optional[str] = ""

    def __init__(self, **_):
        super().__init__(**_)


class Contact(BaseMessageComponent):  # TODO
    type: ComponentType = "Contact"
    _type: str  # type 字段冲突
    id: T.Optional[int] = 0

    def __init__(self, **_):
        super().__init__(**_)


class Location(BaseMessageComponent):  # TODO
    type: ComponentType = "Location"
    lat: float
    lon: float
    title: T.Optional[str] = ""
    content: T.Optional[str] = ""

    def __init__(self, **_):
        super().__init__(**_)


class Music(BaseMessageComponent):
    type: ComponentType = "Music"
    _type: str
    id: T.Optional[int] = 0
    url: T.Optional[str] = ""
    audio: T.Optional[str] = ""
    title: T.Optional[str] = ""
    content: T.Optional[str] = ""
    image: T.Optional[str] = ""

    def __init__(self, **_):
        # for k in _.keys():
        #     if k == "_type" and _[k] not in ["qq", "163", "xm", "custom"]:
        #         logger.warn(f"Protocol: {k}={_[k]} doesn't match values")
        super().__init__(**_)


class Image(BaseMessageComponent):
    type: ComponentType = "Image"
    file: T.Optional[str] = ""
    _type: T.Optional[str] = ""
    subType: T.Optional[int] = 0
    url: T.Optional[str] = ""
    cache: T.Optional[bool] = True
    id: T.Optional[int] = 40000
    c: T.Optional[int] = 2
    # 额外
    path: T.Optional[str] = ""
    file_unique: T.Optional[str] = ""  # 某些平台可能有图片缓存的唯一标识

    def __init__(self, file: T.Optional[str], **_):
        super().__init__(file=file, **_)

    @staticmethod
    def fromURL(url: str, **_):
        if url.startswith("http://") or url.startswith("https://"):
            return Image(file=url, **_)
        raise Exception("not a valid url")

    @staticmethod
    def fromFileSystem(path, **_):
        return Image(file=f"file:///{os.path.abspath(path)}", path=path, **_)

    @staticmethod
    def fromBase64(base64: str, **_):
        return Image(f"base64://{base64}", **_)

    @staticmethod
    def fromBytes(byte: bytes):
        return Image.fromBase64(base64.b64encode(byte).decode())

    @staticmethod
    def fromIO(IO):
        return Image.fromBytes(IO.read())

    async def convert_to_file_path(self) -> str:
        """将这个图片统一转换为本地文件路径。这个方法避免了手动判断图片数据类型，直接返回图片数据的本地路径（如果是网络 URL, 则会自动进行下载）。

        Returns:
            str: 图片的本地路径，以绝对路径表示。
        """
        url = self.url if self.url else self.file
        if url and url.startswith("file:///"):
            image_file_path = url[8:]
            return image_file_path
        elif url and url.startswith("http"):
            image_file_path = await download_image_by_url(url)
            return os.path.abspath(image_file_path)
        elif url and url.startswith("base64://"):
            bs64_data = url.removeprefix("base64://")
            image_bytes = base64.b64decode(bs64_data)
            temp_dir = os.path.join(get_astrbot_data_path(), "temp")
            image_file_path = os.path.join(temp_dir, f"{uuid.uuid4()}.jpg")
            with open(image_file_path, "wb") as f:
                f.write(image_bytes)
            return os.path.abspath(image_file_path)
        elif os.path.exists(url):
            image_file_path = url
            return os.path.abspath(image_file_path)
        else:
            raise Exception(f"not a valid file: {url}")

    async def convert_to_base64(self) -> str:
        """将这个图片统一转换为 base64 编码。这个方法避免了手动判断图片数据类型，直接返回图片数据的 base64 编码。

        Returns:
            str: 图片的 base64 编码，不以 base64:// 或者 data:image/jpeg;base64, 开头。
        """
        # convert to base64
        url = self.url if self.url else self.file
        if url and url.startswith("file:///"):
            bs64_data = file_to_base64(url[8:])
        elif url and url.startswith("http"):
            image_file_path = await download_image_by_url(url)
            bs64_data = file_to_base64(image_file_path)
        elif url and url.startswith("base64://"):
            bs64_data = url
        elif os.path.exists(url):
            bs64_data = file_to_base64(url)
        else:
            raise Exception(f"not a valid file: {url}")
        bs64_data = bs64_data.removeprefix("base64://")
        return bs64_data

    async def register_to_file_service(self) -> str:
        """
        将图片注册到文件服务。

        Returns:
            str: 注册后的URL

        Raises:
            Exception: 如果未配置 callback_api_base
        """
        callback_host = astrbot_config.get("callback_api_base")

        if not callback_host:
            raise Exception("未配置 callback_api_base，文件服务不可用")

        file_path = await self.convert_to_file_path()

        token = await file_token_service.register_file(file_path)

        logger.debug(f"已注册：{callback_host}/api/file/{token}")

        return f"{callback_host}/api/file/{token}"


class Reply(BaseMessageComponent):
    type: ComponentType = "Reply"
    id: T.Union[str, int]
    """所引用的消息 ID"""
    chain: T.Optional[T.List["BaseMessageComponent"]] = []
    """被引用的消息段列表"""
    sender_id: T.Optional[int] | T.Optional[str] = 0
    """被引用的消息对应的发送者的 ID"""
    sender_nickname: T.Optional[str] = ""
    """被引用的消息对应的发送者的昵称"""
    time: T.Optional[int] = 0
    """被引用的消息发送时间"""
    message_str: T.Optional[str] = ""
    """被引用的消息解析后的纯文本消息字符串"""

    text: T.Optional[str] = ""
    """deprecated"""
    qq: T.Optional[int] = 0
    """deprecated"""
    seq: T.Optional[int] = 0
    """deprecated"""

    def __init__(self, **_):
        super().__init__(**_)


class RedBag(BaseMessageComponent):
    type: ComponentType = "RedBag"
    title: str

    def __init__(self, **_):
        super().__init__(**_)


class Poke(BaseMessageComponent):
    type: str = ""
    id: T.Optional[int] = 0
    qq: T.Optional[int] = 0

    def __init__(self, type: str, **_):
        type = f"Poke:{type}"
        super().__init__(type=type, **_)


class Forward(BaseMessageComponent):
    type: ComponentType = "Forward"
    id: str

    def __init__(self, **_):
        super().__init__(**_)


class Node(BaseMessageComponent):
    """群合并转发消息"""

    type: ComponentType = "Node"
    id: T.Optional[int] = 0  # 忽略
    name: T.Optional[str] = ""  # qq昵称
    uin: T.Optional[str] = "0"  # qq号
    content: T.Optional[list[BaseMessageComponent]] = []
    seq: T.Optional[T.Union[str, list]] = ""  # 忽略
    time: T.Optional[int] = 0  # 忽略

    def __init__(self, content: list[BaseMessageComponent], **_):
        if isinstance(content, Node):
            # back
            content = [content]
        super().__init__(content=content, **_)

    async def to_dict(self):
        data_content = []
        for comp in self.content:
            if isinstance(comp, (Image, Record)):
                # For Image and Record segments, we convert them to base64
                bs64 = await comp.convert_to_base64()
                data_content.append(
                    {
                        "type": comp.type.lower(),
                        "data": {"file": f"base64://{bs64}"},
                    }
                )
            elif isinstance(comp, File):
                # For File segments, we need to handle the file differently
                d = await comp.to_dict()
                data_content.append(d)
            elif isinstance(comp, (Node, Nodes)):
                # For Node segments, we recursively convert them to dict
                d = await comp.to_dict()
                data_content.append(d)
            else:
                d = comp.toDict()
                data_content.append(d)
        return {
            "type": "node",
            "data": {
                "user_id": str(self.uin),
                "nickname": self.name,
                "content": data_content,
            },
        }


class Nodes(BaseMessageComponent):
    type: ComponentType = "Nodes"
    nodes: T.List[Node]

    def __init__(self, nodes: T.List[Node], **_):
        super().__init__(nodes=nodes, **_)

    def toDict(self):
        """Deprecated. Use to_dict instead"""
        ret = {
            "messages": [],
        }
        for node in self.nodes:
            d = node.toDict()
            ret["messages"].append(d)
        return ret

    async def to_dict(self):
        """将 Nodes 转换为字典格式，适用于 OneBot JSON 格式"""
        ret = {"messages": []}
        for node in self.nodes:
            d = await node.to_dict()
            ret["messages"].append(d)
        return ret


class Xml(BaseMessageComponent):
    type: ComponentType = "Xml"
    data: str
    resid: T.Optional[int] = 0

    def __init__(self, **_):
        super().__init__(**_)


class Json(BaseMessageComponent):
    type: ComponentType = "Json"
    data: T.Union[str, dict]
    resid: T.Optional[int] = 0

    def __init__(self, data, **_):
        if isinstance(data, dict):
            data = json.dumps(data)
        super().__init__(data=data, **_)


class CardImage(BaseMessageComponent):
    type: ComponentType = "CardImage"
    file: str
    cache: T.Optional[bool] = True
    minwidth: T.Optional[int] = 400
    minheight: T.Optional[int] = 400
    maxwidth: T.Optional[int] = 500
    maxheight: T.Optional[int] = 500
    source: T.Optional[str] = ""
    icon: T.Optional[str] = ""

    def __init__(self, **_):
        super().__init__(**_)

    @staticmethod
    def fromFileSystem(path, **_):
        return CardImage(file=f"file:///{os.path.abspath(path)}", **_)


class TTS(BaseMessageComponent):
    type: ComponentType = "TTS"
    text: str

    def __init__(self, **_):
        super().__init__(**_)


class Unknown(BaseMessageComponent):
    type: ComponentType = "Unknown"
    text: str

    def toString(self):
        return ""


class File(BaseMessageComponent):
    """
    文件消息段
    """

    type: ComponentType = "File"
    name: T.Optional[str] = ""  # 名字
    file_: T.Optional[str] = ""  # 本地路径
    url: T.Optional[str] = ""  # url

    def __init__(self, name: str, file: str = "", url: str = ""):
        """文件消息段。"""
        super().__init__(name=name, file_=file, url=url)

    @property
    def file(self) -> str:
        """
        获取文件路径，如果文件不存在但有URL，则同步下载文件

        Returns:
            str: 文件路径
        """
        if self.file_ and os.path.exists(self.file_):
            return os.path.abspath(self.file_)

        if self.url:
            try:
                loop = asyncio.get_event_loop()
                if loop.is_running():
                    logger.warning(
                        (
                            "不可以在异步上下文中同步等待下载! "
                            "这个警告通常发生于某些逻辑试图通过 <File>.file 获取文件消息段的文件内容。"
                            "请使用 await get_file() 代替直接获取 <File>.file 字段"
                        )
                    )
                    return ""
                else:
                    # 等待下载完成
                    loop.run_until_complete(self._download_file())

                    if self.file_ and os.path.exists(self.file_):
                        return os.path.abspath(self.file_)
            except Exception as e:
                logger.error(f"文件下载失败: {e}")

        return ""

    @file.setter
    def file(self, value: str):
        """
        向前兼容, 设置file属性, 传入的参数可能是文件路径或URL

        Args:
            value (str): 文件路径或URL
        """
        if value.startswith("http://") or value.startswith("https://"):
            self.url = value
        else:
            self.file_ = value

    async def get_file(self, allow_return_url: bool = False) -> str:
        """异步获取文件。请注意在使用后清理下载的文件, 以免占用过多空间

        Args:
            allow_return_url: 是否允许以文件 http 下载链接的形式返回，这允许您自行控制是否需要下载文件。
            注意，如果为 True，也可能返回文件路径。
        Returns:
            str: 文件路径或者 http 下载链接
        """
        if allow_return_url and self.url:
            return self.url

        if self.file_ and os.path.exists(self.file_):
            return os.path.abspath(self.file_)

        if self.url:
            await self._download_file()
            return os.path.abspath(self.file_)

        return ""

    async def _download_file(self):
        """下载文件"""
        download_dir = os.path.join(get_astrbot_data_path(), "temp")
        os.makedirs(download_dir, exist_ok=True)
        file_path = os.path.join(download_dir, f"{uuid.uuid4().hex}")
        await download_file(self.url, file_path)
        self.file_ = os.path.abspath(file_path)

    async def register_to_file_service(self):
        """
        将文件注册到文件服务。

        Returns:
            str: 注册后的URL

        Raises:
            Exception: 如果未配置 callback_api_base
        """
        callback_host = astrbot_config.get("callback_api_base")

        if not callback_host:
            raise Exception("未配置 callback_api_base，文件服务不可用")

        file_path = await self.get_file()

        token = await file_token_service.register_file(file_path)

        logger.debug(f"已注册：{callback_host}/api/file/{token}")

        return f"{callback_host}/api/file/{token}"

    async def to_dict(self):
        """需要和 toDict 区分开，toDict 是同步方法"""
        url_or_path = await self.get_file(allow_return_url=True)
        if url_or_path.startswith("http"):
            payload_file = url_or_path
        elif callback_host := astrbot_config.get("callback_api_base"):
            callback_host = str(callback_host).removesuffix("/")
            token = await file_token_service.register_file(url_or_path)
            payload_file = f"{callback_host}/api/file/{token}"
            logger.debug(f"Generated file callback link: {payload_file}")
        else:
            payload_file = url_or_path
        return {
            "type": "file",
            "data": {
                "name": self.name,
                "file": payload_file,
            },
        }


class WechatEmoji(BaseMessageComponent):
    type: ComponentType = "WechatEmoji"
    md5: T.Optional[str] = ""
    md5_len: T.Optional[int] = 0
    cdnurl: T.Optional[str] = ""

    def __init__(self, **_):
        super().__init__(**_)


ComponentTypes = {
    "plain": Plain,
    "text": Plain,
    "face": Face,
    "record": Record,
    "video": Video,
    "at": At,
    "rps": RPS,
    "dice": Dice,
    "shake": Shake,
    "anonymous": Anonymous,
    "share": Share,
    "contact": Contact,
    "location": Location,
    "music": Music,
    "image": Image,
    "reply": Reply,
    "redbag": RedBag,
    "poke": Poke,
    "forward": Forward,
    "node": Node,
    "nodes": Nodes,
    "xml": Xml,
    "json": Json,
    "cardimage": CardImage,
    "tts": TTS,
    "unknown": Unknown,
    "file": File,
    "WechatEmoji": WechatEmoji,
}



import os

from astrbot.core import logger


def path_Mapping(mappings, srcPath: str) -> str:
    """路径映射处理函数。尝试支援 Windows 和 Linux 的路径映射。
    Args:
        mappings: 映射规则列表
        srcPath: 原路径
    Returns:
        str: 处理后的路径
    """
    for mapping in mappings:
        rule = mapping.split(":")
        if len(rule) == 2:
            from_, to_ = mapping.split(":")
        elif len(rule) > 4 or len(rule) == 1:
            # 切割后大于4个项目，或者只有1个项目，那肯定是错误的，只能是2，3，4个项目
            logger.warning(f"路径映射规则错误: {mapping}")
            continue
        else:
            # rule.len == 3 or 4
            if os.path.exists(rule[0] + ":" + rule[1]):
                # 前面两个项目合并路径存在，说明是本地Window路径。后面一个或两个项目组成的路径本地大概率无法解析，直接拼接
                from_ = rule[0] + ":" + rule[1]
                if len(rule) == 3:
                    to_ = rule[2]
                else:
                    to_ = rule[2] + ":" + rule[3]
            else:
                # 前面两个项目合并路径不存在，说明第一个项目是本地Linux路径，后面一个或两个项目直接拼接。
                from_ = rule[0]
                if len(rule) == 3:
                    to_ = rule[1] + ":" + rule[2]
                else:
                    # 这种情况下存在四个项目，说明规则也是错误的
                    logger.warning(f"路径映射规则错误: {mapping}")
                    continue

        from_ = from_.removesuffix("/")
        from_ = from_.removesuffix("\\")
        to_ = to_.removesuffix("/")
        to_ = to_.removesuffix("\\")
        # logger.debug(f"\t路径映射-规则(处理): {from_} -> {to_}")

        url = srcPath.removeprefix("file://")
        if url.startswith(from_):
            srcPath = url.replace(from_, to_, 1)
            if ":" in srcPath:
                # Windows路径处理
                srcPath = srcPath.replace("/", "\\")
            else:
                has_replaced_processed = False
                if srcPath.startswith("."):
                    # 相对路径处理。如果是相对路径，可能是Linux路径，也可能是Windows路径
                    sign = srcPath[1]
                    # 处理两个点的情况
                    if sign == ".":
                        sign = srcPath[2]
                    if sign == "/":
                        srcPath = srcPath.replace("\\", "/")
                        has_replaced_processed = True
                    elif sign == "\\":
                        srcPath = srcPath.replace("/", "\\")
                        has_replaced_processed = True
                if not has_replaced_processed:
                    # 如果不是相对路径或不能处理，默认按照Linux路径处理
                    srcPath = srcPath.replace("\\", "/")
            logger.info(f"路径映射: {url} -> {srcPath}")
            return srcPath
    return srcPath



import re


class CommandTokens:
    def __init__(self) -> None:
        self.tokens = []
        self.len = 0

    def get(self, idx: int):
        if idx >= self.len:
            return None
        return self.tokens[idx].strip()


class CommandParserMixin:
    def parse_commands(self, message: str):
        cmd_tokens = CommandTokens()
        cmd_tokens.tokens = re.split(r"\s+", message)
        cmd_tokens.len = len(cmd_tokens.tokens)
        return cmd_tokens

    def regex_match(self, message: str, command: str) -> bool:
        return re.search(command, message, re.MULTILINE) is not None



import re
import asyncio
import functools
from typing import List
from .. import Provider, Personality
from ..entities import LLMResponse
from ..func_tool_manager import FuncCall
from astrbot.core.db import BaseDatabase
from ..register import register_provider_adapter
from astrbot.core.message.message_event_result import MessageChain
from .openai_source import ProviderOpenAIOfficial
from astrbot.core import logger, sp
from dashscope import Application


@register_provider_adapter("dashscope", "Dashscope APP 适配器。")
class ProviderDashscope(ProviderOpenAIOfficial):
    def __init__(
        self,
        provider_config: dict,
        provider_settings: dict,
        db_helper: BaseDatabase,
        persistant_history=False,
        default_persona: Personality = None,
    ) -> None:
        Provider.__init__(
            self,
            provider_config,
            provider_settings,
            persistant_history,
            db_helper,
            default_persona,
        )
        self.api_key = provider_config.get("dashscope_api_key", "")
        if not self.api_key:
            raise Exception("阿里云百炼 API Key 不能为空。")
        self.app_id = provider_config.get("dashscope_app_id", "")
        if not self.app_id:
            raise Exception("阿里云百炼 APP ID 不能为空。")
        self.dashscope_app_type = provider_config.get("dashscope_app_type", "")
        if not self.dashscope_app_type:
            raise Exception("阿里云百炼 APP 类型不能为空。")
        self.model_name = "dashscope"
        self.variables: dict = provider_config.get("variables", {})
        self.rag_options: dict = provider_config.get("rag_options", {})
        self.output_reference = self.rag_options.get("output_reference", False)
        self.rag_options = self.rag_options.copy()
        self.rag_options.pop("output_reference", None)

        self.timeout = provider_config.get("timeout", 120)
        if isinstance(self.timeout, str):
            self.timeout = int(self.timeout)

    def has_rag_options(self):
        """判断是否有 RAG 选项

        Returns:
            bool: 是否有 RAG 选项
        """
        if self.rag_options and (
            len(self.rag_options.get("pipeline_ids", [])) > 0
            or len(self.rag_options.get("file_ids", [])) > 0
        ):
            return True
        return False

    async def text_chat(
        self,
        prompt: str,
        session_id: str = None,
        image_urls: List[str] = [],
        func_tool: FuncCall = None,
        contexts: List = None,
        system_prompt: str = None,
        **kwargs,
    ) -> LLMResponse:
        if contexts is None:
            contexts = []
        # 获得会话变量
        payload_vars = self.variables.copy()
        # 动态变量
        session_vars = sp.get("session_variables", {})
        session_var = session_vars.get(session_id, {})
        payload_vars.update(session_var)

        if (
            self.dashscope_app_type in ["agent", "dialog-workflow"]
            and not self.has_rag_options()
        ):
            # 支持多轮对话的
            new_record = {"role": "user", "content": prompt}
            if image_urls:
                logger.warning("阿里云百炼暂不支持图片输入，将自动忽略图片内容。")
            contexts_no_img = await self._remove_image_from_context(contexts)
            context_query = [*contexts_no_img, new_record]
            if system_prompt:
                context_query.insert(0, {"role": "system", "content": system_prompt})
            for part in context_query:
                if "_no_save" in part:
                    del part["_no_save"]
            # 调用阿里云百炼 API
            payload = {
                "app_id": self.app_id,
                "api_key": self.api_key,
                "messages": context_query,
                "biz_params": payload_vars or None,
            }
            partial = functools.partial(
                Application.call,
                **payload,
            )
            response = await asyncio.get_event_loop().run_in_executor(None, partial)
        else:
            # 不支持多轮对话的
            # 调用阿里云百炼 API
            payload = {
                "app_id": self.app_id,
                "prompt": prompt,
                "api_key": self.api_key,
                "biz_params": payload_vars or None,
            }
            if self.rag_options:
                payload["rag_options"] = self.rag_options
            partial = functools.partial(
                Application.call,
                **payload,
            )
            response = await asyncio.get_event_loop().run_in_executor(None, partial)

        logger.debug(f"dashscope resp: {response}")

        if response.status_code != 200:
            logger.error(
                f"阿里云百炼请求失败: request_id={response.request_id}, code={response.status_code}, message={response.message}, 请参考文档：https://help.aliyun.com/zh/model-studio/developer-reference/error-code"
            )
            return LLMResponse(
                role="err",
                result_chain=MessageChain().message(
                    f"阿里云百炼请求失败: message={response.message} code={response.status_code}"
                ),
            )

        output_text = response.output.get("text", "")
        # RAG 引用脚标格式化
        output_text = re.sub(r"<ref>\[(\d+)\]</ref>", r"[\1]", output_text)
        if self.output_reference and response.output.get("doc_references", None):
            ref_str = ""
            for ref in response.output.get("doc_references", []):
                ref_title = (
                    ref.get("title", "")
                    if ref.get("title")
                    else ref.get("doc_name", "")
                )
                ref_str += f"{ref['index_id']}. {ref_title}\n"
            output_text += f"\n\n回答来源:\n{ref_str}"

        llm_response = LLMResponse("assistant")
        llm_response.result_chain = MessageChain().message(output_text)

        return llm_response

    async def text_chat_stream(
        self,
        prompt,
        session_id=None,
        image_urls=...,
        func_tool=None,
        contexts=...,
        system_prompt=None,
        tool_calls_result=None,
        **kwargs,
    ):
        # raise NotImplementedError("This method is not implemented yet.")
        # 调用 text_chat 模拟流式
        llm_response = await self.text_chat(
            prompt=prompt,
            session_id=session_id,
            image_urls=image_urls,
            func_tool=func_tool,
            contexts=contexts,
            system_prompt=system_prompt,
            tool_calls_result=tool_calls_result,
        )
        llm_response.is_chunk = True
        yield llm_response
        llm_response.is_chunk = False
        yield llm_response

    async def forget(self, session_id):
        return True

    async def get_current_key(self):
        return self.api_key

    async def set_key(self, key):
        raise Exception("阿里云百炼 适配器不支持设置 API Key。")

    async def get_models(self):
        return [self.get_model()]

    async def get_human_readable_context(self, session_id, page, page_size):
        raise Exception("暂不支持获得 阿里云百炼 的历史消息记录。")

    async def terminate(self):
        pass



import uuid
import time
import json
import re
import hashlib
import random
import asyncio
from pathlib import Path
from typing import Dict
from xml.sax.saxutils import escape

from httpx import AsyncClient, Timeout
from astrbot.core.config.default import VERSION

from ..entities import ProviderType
from ..provider import TTSProvider
from ..register import register_provider_adapter

TEMP_DIR = Path("data/temp/azure_tts")
TEMP_DIR.mkdir(parents=True, exist_ok=True)

class OTTSProvider:
    def __init__(self, config: Dict):
        self.skey = config["OTTS_SKEY"]
        self.api_url = config["OTTS_URL"]
        self.auth_time_url = config["OTTS_AUTH_TIME"]
        self.time_offset = 0
        self.last_sync_time = 0
        self.timeout = Timeout(10.0)
        self.retry_count = 3
        self.client = None

    async def __aenter__(self):
        self.client = AsyncClient(timeout=self.timeout)
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        if self.client:
            await self.client.aclose()

    async def _sync_time(self):
        try:
            response = await self.client.get(self.auth_time_url)
            response.raise_for_status()
            server_time = int(response.json()["timestamp"])
            local_time = int(time.time())
            self.time_offset = server_time - local_time
            self.last_sync_time = local_time
        except Exception as e:
            if time.time() - self.last_sync_time > 3600:
                raise RuntimeError("时间同步失败") from e

    async def _generate_signature(self) -> str:
        await self._sync_time()
        timestamp = int(time.time()) + self.time_offset
        nonce = "".join(random.choices("abcdefghijklmnopqrstuvwxyz0123456789", k=10))
        path = re.sub(r"^https?://[^/]+", "", self.api_url) or "/"
        return f"{timestamp}-{nonce}-0-{hashlib.md5(f'{path}-{timestamp}-{nonce}-0-{self.skey}'.encode()).hexdigest()}"

    async def get_audio(self, text: str, voice_params: Dict) -> str:
        file_path = TEMP_DIR / f"otts-{uuid.uuid4()}.wav"
        signature = await self._generate_signature()
        for attempt in range(self.retry_count):
            try:
                response = await self.client.post(
                    f"{self.api_url}?sign={signature}",
                    data={
                        "text": text,
                        "voice": voice_params["voice"],
                        "style": voice_params["style"],
                        "role": voice_params["role"],
                        "rate": voice_params["rate"],
                        "volume": voice_params["volume"]
                    },
                    headers={
                        "User-Agent": f"AstrBot/{VERSION}",
                        "UAK": "AstrBot/AzureTTS"
                    }
                )
                response.raise_for_status()
                file_path.parent.mkdir(parents=True, exist_ok=True)
                with file_path.open("wb") as f:
                    async for chunk in response.aiter_bytes(4096):
                        f.write(chunk)
                return str(file_path.resolve())
            except Exception as e:
                if attempt == self.retry_count - 1:
                    raise RuntimeError(f"OTTS请求失败: {str(e)}") from e
                await asyncio.sleep(0.5 * (attempt + 1))

class AzureNativeProvider(TTSProvider):
    def __init__(self, provider_config: dict, provider_settings: dict):
        super().__init__(provider_config, provider_settings)
        self.subscription_key = provider_config.get("azure_tts_subscription_key", "").strip()
        if not re.fullmatch(r"^[a-zA-Z0-9]{32}$", self.subscription_key):
            raise ValueError("无效的Azure订阅密钥")
        self.region = provider_config.get("azure_tts_region", "eastus").strip()
        self.endpoint = f"https://{self.region}.tts.speech.microsoft.com/cognitiveservices/v1"
        self.client = None
        self.token = None
        self.token_expire = 0
        self.voice_params = {
            "voice": provider_config.get("azure_tts_voice", "zh-CN-YunxiaNeural"),
            "style": provider_config.get("azure_tts_style", "cheerful"),
            "role": provider_config.get("azure_tts_role", "Boy"),
            "rate": provider_config.get("azure_tts_rate", "1"),
            "volume": provider_config.get("azure_tts_volume", "100")
        }

    async def __aenter__(self):
        self.client = AsyncClient(headers={
            "User-Agent": f"AstrBot/{VERSION}",
            "Content-Type": "application/ssml+xml",
            "X-Microsoft-OutputFormat": "riff-48khz-16bit-mono-pcm"
        })
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        if self.client:
            await self.client.aclose()

    async def _refresh_token(self):
        token_url = f"https://{self.region}.api.cognitive.microsoft.com/sts/v1.0/issuetoken"
        response = await self.client.post(
            token_url,
            headers={"Ocp-Apim-Subscription-Key": self.subscription_key}
        )
        response.raise_for_status()
        self.token = response.text
        self.token_expire = time.time() + 540

    async def get_audio(self, text: str) -> str:
        if not self.token or time.time() > self.token_expire:
            await self._refresh_token()
        file_path = TEMP_DIR / f"azure-{uuid.uuid4()}.wav"
        ssml = f"""<speak version='1.0' xmlns='http://www.w3.org/2001/10/synthesis'
            xmlns:mstts='http://www.w3.org/2001/mstts' xml:lang='zh-CN'>
            <voice name='{escape(self.voice_params["voice"])}'>
                <mstts:express-as style='{escape(self.voice_params["style"])}'
                    role='{escape(self.voice_params["role"])}'>
                    <prosody rate='{escape(self.voice_params["rate"])}'
                        volume='{escape(self.voice_params["volume"])}'>
                        {escape(text)}
                    </prosody>
                </mstts:express-as>
            </voice>
        </speak>"""
        response = await self.client.post(
            self.endpoint,
            content=ssml,
            headers={
                "Authorization": f"Bearer {self.token}",
                "User-Agent": f"AstrBot/{VERSION}"
            }
        )
        response.raise_for_status()
        file_path.parent.mkdir(parents=True, exist_ok=True)
        with file_path.open("wb") as f:
            for chunk in response.iter_bytes(4096):
                f.write(chunk)
        return str(file_path.resolve())

@register_provider_adapter("azure_tts", "Azure TTS", ProviderType.TEXT_TO_SPEECH)
class AzureTTSProvider(TTSProvider):
    def __init__(self, provider_config: dict, provider_settings: dict):
        super().__init__(provider_config, provider_settings)
        key_value = provider_config.get("azure_tts_subscription_key", "")
        self.provider = self._parse_provider(key_value, provider_config)

    def _parse_provider(self, key_value: str, config: dict) -> TTSProvider:
        if key_value.lower().startswith("other["):
            try:
                match = re.match(r"other\[(.*)\]", key_value, re.DOTALL)
                if not match:
                    raise ValueError("无效的other[...]格式，应形如 other[{...}]")
                json_str = match.group(1).strip()
                otts_config = json.loads(json_str)
                required = {"OTTS_SKEY", "OTTS_URL", "OTTS_AUTH_TIME"}
                if missing := required - otts_config.keys():
                    raise ValueError(f"缺少OTTS参数: {', '.join(missing)}")
                return OTTSProvider(otts_config)
            except json.JSONDecodeError as e:
                error_msg = (
                    f"JSON解析失败，请检查格式（错误位置：行 {e.lineno} 列 {e.colno}）\n"
                    f"错误详情: {e.msg}\n"
                    f"错误上下文: {json_str[max(0, e.pos-30):e.pos+30]}"
                )
                raise ValueError(error_msg) from e
            except KeyError as e:
                raise ValueError(f"配置错误: 缺少必要参数 {e}") from e
        if re.fullmatch(r"^[a-zA-Z0-9]{32}$", key_value):
            return AzureNativeProvider(config, self.provider_settings)
        raise ValueError("订阅密钥格式无效，应为32位字母数字或other[...]格式")

    async def get_audio(self, text: str) -> str:
        if isinstance(self.provider, OTTSProvider):
            async with self.provider as provider:
                return await provider.get_audio(
                    text,
                    {
                        "voice": self.provider_config.get("azure_tts_voice"),
                        "style": self.provider_config.get("azure_tts_style"),
                        "role": self.provider_config.get("azure_tts_role"),
                        "rate": self.provider_config.get("azure_tts_rate"),
                        "volume": self.provider_config.get("azure_tts_volume")
                    }
                )
        else:
            async with self.provider as provider:
                return await provider.get_audio(text)



"""
AstrBot 会话-对话管理器, 维护两个本地存储, 其中一个是 json 格式的shared_preferences, 另外一个是数据库

在 AstrBot 中, 会话和对话是独立的, 会话用于标记对话窗口, 例如群聊"123456789"可以建立一个会话,
在一个会话中可以建立多个对话, 并且支持对话的切换和删除
"""

import uuid
import json
import asyncio
from astrbot.core import sp
from typing import Dict, List
from astrbot.core.db import BaseDatabase
from astrbot.core.db.po import Conversation


class ConversationManager:
    """负责管理会话与 LLM 的对话，某个会话当前正在用哪个对话。"""

    def __init__(self, db_helper: BaseDatabase):
        # session_conversations 字典记录会话ID-对话ID 映射关系
        self.session_conversations: Dict[str, str] = sp.get("session_conversation", {})
        self.db = db_helper
        self.save_interval = 60  # 每 60 秒保存一次
        self._start_periodic_save()

    def _start_periodic_save(self):
        """启动定时保存任务"""
        asyncio.create_task(self._periodic_save())

    async def _periodic_save(self):
        """定时保存会话对话映射关系到存储中"""
        while True:
            await asyncio.sleep(self.save_interval)
            self._save_to_storage()

    def _save_to_storage(self):
        """保存会话对话映射关系到存储中"""
        sp.put("session_conversation", self.session_conversations)

    async def new_conversation(self, unified_msg_origin: str) -> str:
        """新建对话，并将当前会话的对话转移到新对话

        Args:
            unified_msg_origin (str): 统一的消息来源字符串。格式为 platform_name:message_type:session_id
        Returns:
            conversation_id (str): 对话 ID, 是 uuid 格式的字符串
        """
        conversation_id = str(uuid.uuid4())
        self.db.new_conversation(user_id=unified_msg_origin, cid=conversation_id)
        self.session_conversations[unified_msg_origin] = conversation_id
        sp.put("session_conversation", self.session_conversations)
        return conversation_id

    async def switch_conversation(self, unified_msg_origin: str, conversation_id: str):
        """切换会话的对话

        Args:
            unified_msg_origin (str): 统一的消息来源字符串。格式为 platform_name:message_type:session_id
            conversation_id (str): 对话 ID, 是 uuid 格式的字符串
        """
        self.session_conversations[unified_msg_origin] = conversation_id
        sp.put("session_conversation", self.session_conversations)

    async def delete_conversation(
        self, unified_msg_origin: str, conversation_id: str = None
    ):
        """删除会话的对话，当 conversation_id 为 None 时删除会话当前的对话

        Args:
            unified_msg_origin (str): 统一的消息来源字符串。格式为 platform_name:message_type:session_id
            conversation_id (str): 对话 ID, 是 uuid 格式的字符串
        """
        conversation_id = self.session_conversations.get(unified_msg_origin)
        if conversation_id:
            self.db.delete_conversation(user_id=unified_msg_origin, cid=conversation_id)
            del self.session_conversations[unified_msg_origin]
            sp.put("session_conversation", self.session_conversations)

    async def get_curr_conversation_id(self, unified_msg_origin: str) -> str:
        """获取会话当前的对话 ID

        Args:
            unified_msg_origin (str): 统一的消息来源字符串。格式为 platform_name:message_type:session_id
        Returns:
            conversation_id (str): 对话 ID, 是 uuid 格式的字符串
        """
        return self.session_conversations.get(unified_msg_origin, None)

    async def get_conversation(
        self, unified_msg_origin: str, conversation_id: str
    ) -> Conversation:
        """获取会话的对话

        Args:
            unified_msg_origin (str): 统一的消息来源字符串。格式为 platform_name:message_type:session_id
            conversation_id (str): 对话 ID, 是 uuid 格式的字符串
        Returns:
            conversation (Conversation): 对话对象
        """
        return self.db.get_conversation_by_user_id(unified_msg_origin, conversation_id)

    async def get_conversations(self, unified_msg_origin: str) -> List[Conversation]:
        """获取会话的所有对话

        Args:
            unified_msg_origin (str): 统一的消息来源字符串。格式为 platform_name:message_type:session_id
        Returns:
            conversations (List[Conversation]): 对话对象列表
        """
        return self.db.get_conversations(unified_msg_origin)

    async def update_conversation(
        self, unified_msg_origin: str, conversation_id: str, history: List[Dict]
    ):
        """更新会话的对话

        Args:
            unified_msg_origin (str): 统一的消息来源字符串。格式为 platform_name:message_type:session_id
            conversation_id (str): 对话 ID, 是 uuid 格式的字符串
            history (List[Dict]): 对话历史记录, 是一个字典列表, 每个字典包含 role 和 content 字段
        """
        if conversation_id:
            self.db.update_conversation(
                user_id=unified_msg_origin,
                cid=conversation_id,
                history=json.dumps(history),
            )

    async def update_conversation_title(self, unified_msg_origin: str, title: str):
        """更新会话的对话标题

        Args:
            unified_msg_origin (str): 统一的消息来源字符串。格式为 platform_name:message_type:session_id
            title (str): 对话标题
        """
        conversation_id = self.session_conversations.get(unified_msg_origin)
        if conversation_id:
            self.db.update_conversation_title(
                user_id=unified_msg_origin, cid=conversation_id, title=title
            )

    async def update_conversation_persona_id(
        self, unified_msg_origin: str, persona_id: str
    ):
        """更新会话的对话 Persona ID

        Args:
            unified_msg_origin (str): 统一的消息来源字符串。格式为 platform_name:message_type:session_id
            persona_id (str): 对话 Persona ID
        """
        conversation_id = self.session_conversations.get(unified_msg_origin)
        if conversation_id:
            self.db.update_conversation_persona_id(
                user_id=unified_msg_origin, cid=conversation_id, persona_id=persona_id
            )

    async def get_human_readable_context(
        self, unified_msg_origin, conversation_id, page=1, page_size=10
    ):
        """获取人类可读的上下文

        Args:
            unified_msg_origin (str): 统一的消息来源字符串。格式为 platform_name:message_type:session_id
            conversation_id (str): 对话 ID, 是 uuid 格式的字符串
            page (int): 页码
            page_size (int): 每页大小
        """
        conversation = await self.get_conversation(unified_msg_origin, conversation_id)
        history = json.loads(conversation.history)

        contexts = []
        temp_contexts = []
        for record in history:
            if record["role"] == "user":
                temp_contexts.append(f"User: {record['content']}")
            elif record["role"] == "assistant":
                if "content" in record and record["content"]:
                    temp_contexts.append(f"Assistant: {record['content']}")
                elif "tool_calls" in record:
                    tool_calls_str = json.dumps(
                        record["tool_calls"], ensure_ascii=False
                    )
                    temp_contexts.append(f"Assistant: [函数调用] {tool_calls_str}")
                else:
                    temp_contexts.append("Assistant: [未知的内容]")
                contexts.insert(0, temp_contexts)
                temp_contexts = []

        # 展平 contexts 列表
        contexts = [item for sublist in contexts for item in sublist]

        # 计算分页
        paged_contexts = contexts[(page - 1) * page_size : page * page_size]
        total_pages = len(contexts) // page_size
        if len(contexts) % page_size != 0:
            total_pages += 1

        return paged_contexts, total_pages



from astrbot.core.config.astrbot_config import AstrBotConfig
from astrbot import logger
from astrbot.core import html_renderer
from astrbot.core.star.register import register_llm_tool as llm_tool

# event
from astrbot.core.message.message_event_result import (
    MessageEventResult,
    MessageChain,
    CommandResult,
    EventResultType,
)
from astrbot.core.platform import AstrMessageEvent

# star register
from astrbot.core.star.register import (
    register_command as command,
    register_command_group as command_group,
    register_event_message_type as event_message_type,
    register_regex as regex,
    register_platform_adapter_type as platform_adapter_type,
)
from astrbot.core.star.filter.event_message_type import (
    EventMessageTypeFilter,
    EventMessageType,
)
from astrbot.core.star.filter.platform_adapter_type import (
    PlatformAdapterTypeFilter,
    PlatformAdapterType,
)
from astrbot.core.star.register import (
    register_star as register,  # 注册插件（Star）
)
from astrbot.core.star import Context, Star
from astrbot.core.star.config import *


# provider
from astrbot.core.provider import Provider, Personality, ProviderMetaData

# platform
from astrbot.core.platform import (
    AstrMessageEvent,
    Platform,
    AstrBotMessage,
    MessageMember,
    MessageType,
    PlatformMetadata,
)

from astrbot.core.platform.register import register_platform_adapter

from .message_components import *


import os
import uuid
import base64
from astrbot.api import logger
from astrbot.api.event import AstrMessageEvent, MessageChain
from astrbot.api.message_components import Plain, Image, Record
from astrbot.core.utils.io import download_image_by_url
from astrbot.core import web_chat_back_queue
from astrbot.core.utils.astrbot_path import get_astrbot_data_path

imgs_dir = os.path.join(get_astrbot_data_path(), "webchat", "imgs")


class WebChatMessageEvent(AstrMessageEvent):
    def __init__(self, message_str, message_obj, platform_meta, session_id):
        super().__init__(message_str, message_obj, platform_meta, session_id)
        os.makedirs(imgs_dir, exist_ok=True)

    @staticmethod
    async def _send(message: MessageChain, session_id: str, streaming: bool = False):
        if not message:
            await web_chat_back_queue.put(
                {"type": "end", "data": "", "streaming": False}
            )
            return ""

        cid = session_id.split("!")[-1]
        data = ""
        for comp in message.chain:
            if isinstance(comp, Plain):
                data = comp.text
                await web_chat_back_queue.put(
                    {
                        "type": "plain",
                        "cid": cid,
                        "data": data,
                        "streaming": streaming,
                    }
                )
            elif isinstance(comp, Image):
                # save image to local
                filename = str(uuid.uuid4()) + ".jpg"
                path = os.path.join(imgs_dir, filename)
                if comp.file and comp.file.startswith("file:///"):
                    ph = comp.file[8:]
                    with open(path, "wb") as f:
                        with open(ph, "rb") as f2:
                            f.write(f2.read())
                elif comp.file.startswith("base64://"):
                    base64_str = comp.file[9:]
                    image_data = base64.b64decode(base64_str)
                    with open(path, "wb") as f:
                        f.write(image_data)
                elif comp.file and comp.file.startswith("http"):
                    await download_image_by_url(comp.file, path=path)
                else:
                    with open(path, "wb") as f:
                        with open(comp.file, "rb") as f2:
                            f.write(f2.read())
                data = f"[IMAGE]{filename}"
                await web_chat_back_queue.put(
                    {
                        "type": "image",
                        "cid": cid,
                        "data": data,
                        "streaming": streaming,
                    }
                )
            elif isinstance(comp, Record):
                # save record to local
                filename = str(uuid.uuid4()) + ".wav"
                path = os.path.join(imgs_dir, filename)
                if comp.file and comp.file.startswith("file:///"):
                    ph = comp.file[8:]
                    with open(path, "wb") as f:
                        with open(ph, "rb") as f2:
                            f.write(f2.read())
                elif comp.file and comp.file.startswith("http"):
                    await download_image_by_url(comp.file, path=path)
                else:
                    with open(path, "wb") as f:
                        with open(comp.file, "rb") as f2:
                            f.write(f2.read())
                data = f"[RECORD]{filename}"
                await web_chat_back_queue.put(
                    {
                        "type": "record",
                        "cid": cid,
                        "data": data,
                        "streaming": streaming,
                    }
                )
            else:
                logger.debug(f"webchat 忽略: {comp.type}")

        return data

    async def send(self, message: MessageChain):
        await WebChatMessageEvent._send(message, session_id=self.session_id)
        await web_chat_back_queue.put(
            {
                "type": "end",
                "data": "",
                "streaming": False,
                "cid": self.session_id.split("!")[-1],
            }
        )
        await super().send(message)

    async def send_streaming(self, generator, use_fallback: bool = False):
        final_data = ""
        async for chain in generator:
            final_data += await WebChatMessageEvent._send(
                chain, session_id=self.session_id, streaming=True
            )

        await web_chat_back_queue.put(
            {
                "type": "end",
                "data": final_data,
                "streaming": True,
                "cid": self.session_id.split("!")[-1],
            }
        )
        await super().send_streaming(generator, use_fallback)



from ..stage import Stage, register_stage
from ..context import PipelineContext
from astrbot import logger
from typing import Union, AsyncGenerator
from astrbot.core.platform.astr_message_event import AstrMessageEvent
from astrbot.core.message.message_event_result import MessageEventResult, MessageChain
from astrbot.core.message.components import At
from astrbot.core.star.star_handler import star_handlers_registry, EventType
from astrbot.core.star.star import star_map
from astrbot.core.star.filter.permission import PermissionTypeFilter


@register_stage
class WakingCheckStage(Stage):
    """检查是否需要唤醒。唤醒机器人有如下几点条件：

    1. 机器人被 @ 了
    2. 机器人的消息被提到了
    3. 以 wake_prefix 前缀开头，并且消息没有以 At 消息段开头
    4. 插件（Star）的 handler filter 通过
    5. 私聊情况下，位于 admins_id 列表中的管理员的消息（在白名单阶段中）
    """

    async def initialize(self, ctx: PipelineContext) -> None:
        """初始化唤醒检查阶段

        Args:
            ctx (PipelineContext): 消息管道上下文对象, 包括配置和插件管理器
        """
        self.ctx = ctx
        self.no_permission_reply = self.ctx.astrbot_config["platform_settings"].get(
            "no_permission_reply", True
        )
        # 私聊是否需要 wake_prefix 才能唤醒机器人
        self.friend_message_needs_wake_prefix = self.ctx.astrbot_config[
            "platform_settings"
        ].get("friend_message_needs_wake_prefix", False)
        # 是否忽略机器人自己发送的消息
        self.ignore_bot_self_message = self.ctx.astrbot_config["platform_settings"].get(
            "ignore_bot_self_message", False
        )

    async def process(
        self, event: AstrMessageEvent
    ) -> Union[None, AsyncGenerator[None, None]]:
        if (
            self.ignore_bot_self_message
            and event.get_self_id() == event.get_sender_id()
        ):
            # 忽略机器人自己发送的消息
            event.stop_event()
            return
        # 设置 sender 身份
        event.message_str = event.message_str.strip()
        for admin_id in self.ctx.astrbot_config["admins_id"]:
            if str(event.get_sender_id()) == admin_id:
                event.role = "admin"
                break

        # 检查 wake
        wake_prefixes = self.ctx.astrbot_config["wake_prefix"]
        messages = event.get_messages()
        is_wake = False
        for wake_prefix in wake_prefixes:
            if event.message_str.startswith(wake_prefix):
                if (
                    not event.is_private_chat()
                    and isinstance(messages[0], At)
                    and str(messages[0].qq) != str(event.get_self_id())
                    and str(messages[0].qq) != "all"
                ):
                    # 如果是群聊，且第一个消息段是 At 消息，但不是 At 机器人或 At 全体成员，则不唤醒
                    break
                is_wake = True
                event.is_at_or_wake_command = True
                event.is_wake = True
                event.message_str = event.message_str[len(wake_prefix) :].strip()
                break
        if not is_wake:
            # 检查是否有 at 消息
            for message in messages:
                if isinstance(message, At) and (
                    str(message.qq) == str(event.get_self_id())
                    or str(message.qq) == "all"
                ):
                    is_wake = True
                    event.is_wake = True
                    wake_prefix = ""
                    event.is_at_or_wake_command = True
                    break
            # 检查是否是私聊
            if event.is_private_chat() and not self.friend_message_needs_wake_prefix:
                is_wake = True
                event.is_wake = True
                event.is_at_or_wake_command = True
                wake_prefix = ""

        # 检查插件的 handler filter
        activated_handlers = []
        handlers_parsed_params = {}  # 注册了指令的 handler

        for handler in star_handlers_registry.get_handlers_by_event_type(
            EventType.AdapterMessageEvent
        ):
            # filter 需满足 AND 逻辑关系
            passed = True
            permission_not_pass = False
            permission_filter_raise_error = False
            if len(handler.event_filters) == 0:
                continue

            for filter in handler.event_filters:
                try:
                    if isinstance(filter, PermissionTypeFilter):
                        if not filter.filter(event, self.ctx.astrbot_config):
                            permission_not_pass = True
                            permission_filter_raise_error = filter.raise_error
                    else:
                        if not filter.filter(event, self.ctx.astrbot_config):
                            passed = False
                            break
                except Exception as e:
                    await event.send(
                        MessageEventResult().message(
                            f"插件 {star_map[handler.handler_module_path].name}: {e}"
                        )
                    )
                    await event._post_send()
                    event.stop_event()
                    passed = False
                    break
            if passed:
                if permission_not_pass:
                    if not permission_filter_raise_error:
                        # 跳过
                        continue
                    if self.no_permission_reply:
                        await event.send(
                            MessageChain().message(
                                f"您(ID: {event.get_sender_id()})的权限不足以使用此指令。通过 /sid 获取 ID 并请管理员添加。"
                            )
                        )
                        await event._post_send()
                    logger.info(
                        f"触发 {star_map[handler.handler_module_path].name} 时, 用户(ID={event.get_sender_id()}) 权限不足。"
                    )
                    event.stop_event()
                    return

                is_wake = True
                event.is_wake = True

                activated_handlers.append(handler)
                if "parsed_params" in event.get_extra():
                    handlers_parsed_params[handler.handler_full_name] = event.get_extra(
                        "parsed_params"
                    )

            event.clear_extra()

        event.set_extra("activated_handlers", activated_handlers)
        event.set_extra("handlers_parsed_params", handlers_parsed_params)

        if not is_wake:
            event.stop_event()



import asyncio
import base64
import io
from typing import TYPE_CHECKING

import aiohttp
from PIL import Image as PILImage  # 使用别名避免冲突

from astrbot import logger
from astrbot.core.message.components import (
    Image,
    Plain,
    WechatEmoji,
    Record,
)  # Import Image
from astrbot.core.message.message_event_result import MessageChain
from astrbot.core.platform.astr_message_event import AstrMessageEvent
from astrbot.core.platform.astrbot_message import AstrBotMessage, MessageType
from astrbot.core.platform.platform_metadata import PlatformMetadata
from astrbot.core.utils.tencent_record_helper import wav_to_tencent_silk_base64

if TYPE_CHECKING:
    from .wechatpadpro_adapter import WeChatPadProAdapter


class WeChatPadProMessageEvent(AstrMessageEvent):
    def __init__(
        self,
        message_str: str,
        message_obj: AstrBotMessage,
        platform_meta: PlatformMetadata,
        session_id: str,
        adapter: "WeChatPadProAdapter",  # 传递适配器实例
    ):
        super().__init__(message_str, message_obj, platform_meta, session_id)
        self.message_obj = message_obj  # Save the full message object
        self.adapter = adapter  # Save the adapter instance

    async def send(self, message: MessageChain):
        async with aiohttp.ClientSession() as session:
            for comp in message.chain:
                await asyncio.sleep(1)
                if isinstance(comp, Plain):
                    await self._send_text(session, comp.text)
                elif isinstance(comp, Image):
                    await self._send_image(session, comp)
                elif isinstance(comp, WechatEmoji):
                    await self._send_emoji(session, comp)
                elif isinstance(comp, Record):
                    await self._send_voice(session, comp)
        await super().send(message)

    async def _send_image(self, session: aiohttp.ClientSession, comp: Image):
        b64 = await comp.convert_to_base64()
        raw = self._validate_base64(b64)
        b64c = self._compress_image(raw)
        payload = {
            "MsgItem": [
                {"ImageContent": b64c, "MsgType": 3, "ToUserName": self.session_id}
            ]
        }
        url = f"{self.adapter.base_url}/message/SendImageNewMessage"
        await self._post(session, url, payload)

    async def _send_text(self, session: aiohttp.ClientSession, text: str):
        if (
            self.message_obj.type == MessageType.GROUP_MESSAGE  # 确保是群聊消息
            and self.adapter.settings.get(
                "reply_with_mention", False
            )  # 检查适配器设置是否启用 reply_with_mention
            and self.message_obj.sender  # 确保有发送者信息
            and (
                self.message_obj.sender.user_id or self.message_obj.sender.nickname
            )  # 确保发送者有 ID 或昵称
        ):
            # 优先使用 nickname，如果没有则使用 user_id
            mention_text = (
                self.message_obj.sender.nickname or self.message_obj.sender.user_id
            )
            message_text = f"@{mention_text} {text}"
            # logger.info(f"已添加 @ 信息: {message_text}")
        else:
            message_text = text
        payload = {
            "MsgItem": [
                {
                    "MsgType": 1,
                    "TextContent": message_text,
                    "ToUserName": self.session_id,
                }
            ]
        }
        url = f"{self.adapter.base_url}/message/SendTextMessage"
        await self._post(session, url, payload)

    async def _send_emoji(self, session: aiohttp.ClientSession, comp: WechatEmoji):
        payload = {
            "EmojiList": [
                {
                    "EmojiMd5": comp.md5,
                    "EmojiSize": comp.md5_len,
                    "ToUserName": self.session_id,
                }
            ]
        }
        url = f"{self.adapter.base_url}/message/SendEmojiMessage"
        await self._post(session, url, payload)

    async def _send_voice(self, session: aiohttp.ClientSession, comp: Record):
        record_path = await comp.convert_to_file_path()
        # 默认已经存在 data/temp 中
        b64, duration = await wav_to_tencent_silk_base64(record_path)
        payload = {
            "ToUserName": self.session_id,
            "VoiceData": b64,
            "VoiceFormat": 4,
            "VoiceSecond": duration,
        }
        url = f"{self.adapter.base_url}/message/SendVoice"
        await self._post(session, url, payload)

    @staticmethod
    def _validate_base64(b64: str) -> bytes:
        return base64.b64decode(b64, validate=True)

    @staticmethod
    def _compress_image(data: bytes) -> str:
        img = PILImage.open(io.BytesIO(data))
        buf = io.BytesIO()
        if img.format == "JPEG":
            img.save(buf, "JPEG", quality=80)
        else:
            if img.mode in ("RGBA", "P"):
                img = img.convert("RGB")
            img.save(buf, "JPEG", quality=80)
        # logger.info("图片处理完成！！！")
        return base64.b64encode(buf.getvalue()).decode()

    async def _post(self, session, url, payload):
        params = {"key": self.adapter.auth_key}
        try:
            async with session.post(url, params=params, json=payload) as resp:
                data = await resp.json()
                if resp.status != 200 or data.get("Code") != 200:
                    logger.error(f"{url} failed: {resp.status} {data}")
        except Exception as e:
            logger.error(f"{url} error: {e}")


# TODO: 添加对其他消息组件类型的处理 (Record, Video, At等)
# elif isinstance(component, Record):
#     pass
# elif isinstance(component, Video):
#     pass
# elif isinstance(component, At):
#     pass
# ...



import time
import asyncio
import uuid
import os
from typing import Awaitable, Any
from astrbot.core.platform import (
    Platform,
    AstrBotMessage,
    MessageMember,
    MessageType,
    PlatformMetadata,
)
from astrbot.core.message.message_event_result import MessageChain
from astrbot.core.message.components import Plain, Image, Record  # noqa: F403
from astrbot import logger
from astrbot.core import web_chat_queue
from .webchat_event import WebChatMessageEvent
from astrbot.core.platform.astr_message_event import MessageSesion
from ...register import register_platform_adapter
from astrbot.core.utils.astrbot_path import get_astrbot_data_path


class QueueListener:
    def __init__(self, queue: asyncio.Queue, callback: callable) -> None:
        self.queue = queue
        self.callback = callback

    async def run(self):
        while True:
            data = await self.queue.get()
            await self.callback(data)


@register_platform_adapter("webchat", "webchat")
class WebChatAdapter(Platform):
    def __init__(
        self, platform_config: dict, platform_settings: dict, event_queue: asyncio.Queue
    ) -> None:
        super().__init__(event_queue)

        self.config = platform_config
        self.settings = platform_settings
        self.unique_session = platform_settings["unique_session"]
        self.imgs_dir = os.path.join(get_astrbot_data_path(), "webchat", "imgs")
        os.makedirs(self.imgs_dir, exist_ok=True)

        self.metadata = PlatformMetadata(
            name="webchat", description="webchat", id=self.config.get("id")
        )

    async def send_by_session(
        self, session: MessageSesion, message_chain: MessageChain
    ):
        await WebChatMessageEvent._send(message_chain, session.session_id)
        await super().send_by_session(session, message_chain)

    async def convert_message(self, data: tuple) -> AstrBotMessage:
        username, cid, payload = data

        abm = AstrBotMessage()
        abm.self_id = "webchat"
        abm.tag = "webchat"
        abm.sender = MessageMember(username, username)

        abm.type = MessageType.FRIEND_MESSAGE

        abm.session_id = f"webchat!{username}!{cid}"

        abm.message_id = str(uuid.uuid4())
        abm.message = []

        if payload["message"]:
            abm.message.append(Plain(payload["message"]))
        if payload["image_url"]:
            if isinstance(payload["image_url"], list):
                for img in payload["image_url"]:
                    abm.message.append(
                        Image.fromFileSystem(os.path.join(self.imgs_dir, img))
                    )
            else:
                abm.message.append(
                    Image.fromFileSystem(
                        os.path.join(self.imgs_dir, payload["image_url"])
                    )
                )
        if payload["audio_url"]:
            if isinstance(payload["audio_url"], list):
                for audio in payload["audio_url"]:
                    path = os.path.join(self.imgs_dir, audio)
                    abm.message.append(Record(file=path, path=path))
            else:
                path = os.path.join(self.imgs_dir, payload["audio_url"])
                abm.message.append(Record(file=path, path=path))

        logger.debug(f"WebChatAdapter: {abm.message}")

        message_str = payload["message"]
        abm.timestamp = int(time.time())
        abm.message_str = message_str
        abm.raw_message = data
        return abm

    def run(self) -> Awaitable[Any]:
        async def callback(data: tuple):
            abm = await self.convert_message(data)
            await self.handle_msg(abm)

        bot = QueueListener(web_chat_queue, callback)
        return bot.run()

    def meta(self) -> PlatformMetadata:
        return self.metadata

    async def handle_msg(self, message: AstrBotMessage):
        message_event = WebChatMessageEvent(
            message_str=message.message_str,
            message_obj=message,
            platform_meta=self.meta(),
            session_id=message.session_id,
        )

        self.commit_event(message_event)

    async def terminate(self):
        # Do nothing
        pass



from astrbot import logger
import aiohttp
import json


class GeweDownloader:
    def __init__(self, base_url: str, download_base_url: str, token: str):
        self.base_url = base_url
        self.download_base_url = download_base_url
        self.headers = {"Content-Type": "application/json", "X-GEWE-TOKEN": token}

    async def _post_json(self, baseurl: str, route: str, payload: dict):
        async with aiohttp.ClientSession() as session:
            async with session.post(
                f"{baseurl}{route}", headers=self.headers, json=payload
            ) as resp:
                return await resp.read()

    async def download_voice(self, appid: str, xml: str, msg_id: str):
        payload = {"appId": appid, "xml": xml, "msgId": msg_id}
        return await self._post_json(self.base_url, "/message/downloadVoice", payload)

    async def download_image(self, appid: str, xml: str) -> str:
        """返回一个可下载的 URL"""
        choices = [2, 3]  #  2:常规图片 3:缩略图

        for choice in choices:
            try:
                payload = {"appId": appid, "xml": xml, "type": choice}
                data = await self._post_json(
                    self.base_url, "/message/downloadImage", payload
                )
                json_blob = json.loads(data)
                if "fileUrl" in json_blob["data"]:
                    return self.download_base_url + json_blob["data"]["fileUrl"]

            except BaseException as e:
                logger.error(f"gewe download image: {e}")
                continue

        raise Exception("无法下载图片")

    async def download_emoji_md5(self, app_id, emoji_md5):
        """下载emoji"""
        try:
            payload = {"appId": app_id, "emojiMd5": emoji_md5}

            # gewe 计划中的接口，暂时没有实现。返回代码404
            data = await self._post_json(
                self.base_url, "/message/downloadEmojiMd5", payload
            )
            json_blob = json.loads(data)
            return json_blob
        except BaseException as e:
            logger.error(f"gewe download emoji: {e}")



import json
import os
import uuid
import aiohttp
from typing import Dict, List, Union, AsyncIterator
from astrbot.core.utils.astrbot_path import get_astrbot_data_path
from astrbot.api import logger
from ..entities import ProviderType
from ..provider import TTSProvider
from ..register import register_provider_adapter


@register_provider_adapter(
    "minimax_tts_api", "MiniMax TTS API", provider_type=ProviderType.TEXT_TO_SPEECH
)
class ProviderMiniMaxTTSAPI(TTSProvider):
    def __init__(
        self,
        provider_config: dict,
        provider_settings: dict,
    ) -> None:
        super().__init__(provider_config, provider_settings)
        self.chosen_api_key: str = provider_config.get("api_key", "")
        self.api_base: str = provider_config.get(
            "api_base", "https://api.minimax.chat/v1/t2a_v2"
        )
        self.group_id: str = provider_config.get("minimax-group-id", "")
        self.set_model(provider_config.get("model", ""))
        self.lang_boost: str = provider_config.get("minimax-langboost", "auto")
        self.is_timber_weight: bool = provider_config.get(
            "minimax-is-timber-weight", False
        )
        self.timber_weight: List[Dict[str, Union[str, int]]] = json.loads(
            provider_config.get(
                "minimax-timber-weight",
                '[{"voice_id": "Chinese (Mandarin)_Warm_Girl", "weight": 1}]',
            )
        )

        self.voice_setting: dict = {
            "speed": provider_config.get("minimax-voice-speed", 1.0),
            "vol": provider_config.get("minimax-voice-vol", 1.0),
            "pitch": provider_config.get("minimax-voice-pitch", 0),
            "voice_id": ""
            if self.is_timber_weight
            else provider_config.get("minimax-voice-id", ""),
            "emotion": provider_config.get("minimax-voice-emotion", "neutral"),
            "latex_read": provider_config.get("minimax-voice-latex", False),
            "english_normalization": provider_config.get(
                "minimax-voice-english-normalization", False
            ),
        }

        self.audio_setting: dict = {
            "sample_rate": 32000,
            "bitrate": 128000,
            "format": "mp3",
        }

        self.concat_base_url: str = f"{self.api_base}?GroupId={self.group_id}"
        self.headers = {
            "Authorization": f"Bearer {self.chosen_api_key}",
            "accept": "application/json, text/plain, */*",
            "content-type": "application/json",
        }

    def _build_tts_stream_body(self, text: str):
        """构建流式请求体"""
        dict_body: Dict[str, object] = {
            "model": self.model_name,
            "text": text,
            "stream": True,
            "language_boost": self.lang_boost,
            "voice_setting": self.voice_setting,
            "audio_setting": self.audio_setting,
        }
        if self.is_timber_weight:
            dict_body["timber_weights"] = self.timber_weight

        return json.dumps(dict_body)

    async def _call_tts_stream(self, text: str) -> AsyncIterator[bytes]:
        """进行流式请求"""
        try:
            async with aiohttp.ClientSession() as session:
                async with session.post(
                    self.concat_base_url,
                    headers=self.headers,
                    data=self._build_tts_stream_body(text),
                    timeout=aiohttp.ClientTimeout(total=60),
                ) as response:
                    response.raise_for_status()

                    buffer = b""
                    while True:
                        chunk = await response.content.read(8192)
                        if not chunk:
                            break

                        buffer += chunk

                        while b"\n\n" in buffer:
                            try:
                                message, buffer = buffer.split(b"\n\n", 1)
                                if message.startswith(b"data: "):
                                    try:
                                        data = json.loads(message[6:])
                                        if "extra_info" in data:
                                            continue
                                        audio = data.get("data", {}).get("audio")
                                        if audio is not None:
                                            yield audio
                                    except json.JSONDecodeError:
                                        logger.warning(
                                            "Failed to parse JSON data from SSE message"
                                        )
                                        continue
                            except ValueError:
                                buffer = buffer[-1024:]

        except aiohttp.ClientError as e:
            raise Exception(f"MiniMax TTS API请求失败: {str(e)}")

    async def _audio_play(self, audio_stream: AsyncIterator[str]) -> bytes:
        """解码数据流到 audio 比特流"""
        chunks = []
        async for chunk in audio_stream:
            if chunk.strip():
                chunks.append(bytes.fromhex(chunk.strip()))
        return b"".join(chunks)

    async def get_audio(self, text: str) -> str:
        temp_dir = os.path.join(get_astrbot_data_path(), "temp")
        os.makedirs(temp_dir, exist_ok=True)
        path = os.path.join(temp_dir, f"minimax_tts_api_{uuid.uuid4()}.mp3")

        try:
            # 直接将异步生成器传递给 _audio_play 方法
            audio_stream = self._call_tts_stream(text)
            audio = await self._audio_play(audio_stream)

            # 结果保存至文件
            with open(path, "wb") as file:
                file.write(audio)

            return path

        except aiohttp.ClientError as e:
            raise e



from astrbot.api.platform import AstrBotMessage, PlatformMetadata
from botpy import Client
from ..qqofficial.qqofficial_message_event import QQOfficialMessageEvent


class QQOfficialWebhookMessageEvent(QQOfficialMessageEvent):
    def __init__(
        self,
        message_str: str,
        message_obj: AstrBotMessage,
        platform_meta: PlatformMetadata,
        session_id: str,
        bot: Client,
    ):
        super().__init__(message_str, message_obj, platform_meta, session_id, bot)



import enum
from . import HandlerFilter
from astrbot.core.platform.astr_message_event import AstrMessageEvent
from astrbot.core.config import AstrBotConfig
from typing import Union


class PlatformAdapterType(enum.Flag):
    AIOCQHTTP = enum.auto()
    QQOFFICIAL = enum.auto()
    VCHAT = enum.auto()
    GEWECHAT = enum.auto()
    TELEGRAM = enum.auto()
    WECOM = enum.auto()
    LARK = enum.auto()
    ALL = AIOCQHTTP | QQOFFICIAL | VCHAT | GEWECHAT | TELEGRAM | WECOM | LARK


ADAPTER_NAME_2_TYPE = {
    "aiocqhttp": PlatformAdapterType.AIOCQHTTP,
    "qq_official": PlatformAdapterType.QQOFFICIAL,
    "vchat": PlatformAdapterType.VCHAT,
    "gewechat": PlatformAdapterType.GEWECHAT,
    "telegram": PlatformAdapterType.TELEGRAM,
    "wecom": PlatformAdapterType.WECOM,
    "lark": PlatformAdapterType.LARK,
}


class PlatformAdapterTypeFilter(HandlerFilter):
    def __init__(self, platform_adapter_type_or_str: Union[PlatformAdapterType, str]):
        self.type_or_str = platform_adapter_type_or_str

    def filter(self, event: AstrMessageEvent, cfg: AstrBotConfig) -> bool:
        adapter_name = event.get_platform_name()
        if adapter_name in ADAPTER_NAME_2_TYPE:
            return ADAPTER_NAME_2_TYPE[adapter_name] & self.type_or_str
        return False



import asyncio
import base64
import datetime
import os
import re
import uuid
import threading

import aiohttp
import anyio
import quart

from astrbot.api import logger, sp
from astrbot.api.message_components import Plain, Image, At, Record, Video
from astrbot.api.platform import AstrBotMessage, MessageMember, MessageType
from astrbot.core.utils.io import download_image_by_url
from .downloader import GeweDownloader
from astrbot.core.utils.astrbot_path import get_astrbot_data_path

try:
    from .xml_data_parser import GeweDataParser
except (ImportError, ModuleNotFoundError) as e:
    logger.warning(
        f"警告: 可能未安装 defusedxml 依赖库，将导致无法解析微信的 表情包、引用 类型的消息: {str(e)}"
    )


class SimpleGewechatClient:
    """针对 Gewechat 的简单实现。

    @author: Soulter
    @website: https://github.com/Soulter
    """

    def __init__(
        self,
        base_url: str,
        nickname: str,
        host: str,
        port: int,
        event_queue: asyncio.Queue,
    ):
        self.base_url = base_url
        if self.base_url.endswith("/"):
            self.base_url = self.base_url[:-1]

        self.download_base_url = self.base_url.split(":")[:-1]  # 去掉端口
        self.download_base_url = ":".join(self.download_base_url) + ":2532/download/"

        self.base_url += "/v2/api"

        logger.info(f"Gewechat API: {self.base_url}")
        logger.info(f"Gewechat 下载 API: {self.download_base_url}")

        if isinstance(port, str):
            port = int(port)

        self.token = None
        self.headers = {}
        self.nickname = nickname
        self.appid = sp.get(f"gewechat-appid-{nickname}", "")

        self.server = quart.Quart(__name__)
        self.server.add_url_rule(
            "/astrbot-gewechat/callback", view_func=self._callback, methods=["POST"]
        )
        self.server.add_url_rule(
            "/astrbot-gewechat/file/<file_token>",
            view_func=self._handle_file,
            methods=["GET"],
        )

        self.host = host
        self.port = port
        self.callback_url = f"http://{self.host}:{self.port}/astrbot-gewechat/callback"
        self.file_server_url = f"http://{self.host}:{self.port}/astrbot-gewechat/file"

        self.event_queue = event_queue

        self.multimedia_downloader = None

        self.userrealnames = {}

        self.shutdown_event = asyncio.Event()

        self.staged_files = {}
        """存储了允许外部访问的文件列表。auth_token: file_path。通过 register_file 方法注册。"""

        self.lock = asyncio.Lock()

    async def get_token_id(self):
        """获取 Gewechat Token。"""
        async with aiohttp.ClientSession() as session:
            async with session.post(f"{self.base_url}/tools/getTokenId") as resp:
                json_blob = await resp.json()
                self.token = json_blob["data"]
                logger.info(f"获取到 Gewechat Token: {self.token}")
                self.headers = {"X-GEWE-TOKEN": self.token}

    async def _convert(self, data: dict) -> AstrBotMessage:
        if "TypeName" in data:
            type_name = data["TypeName"]
        elif "type_name" in data:
            type_name = data["type_name"]
        else:
            raise Exception("无法识别的消息类型")

        # 以下没有业务处理，只是避免控制台打印太多的日志
        if type_name == "ModContacts":
            logger.info("gewechat下发：ModContacts消息通知。")
            return
        if type_name == "DelContacts":
            logger.info("gewechat下发：DelContacts消息通知。")
            return

        if type_name == "Offline":
            logger.critical("收到 gewechat 下线通知。")
            return

        d = None
        if "Data" in data:
            d = data["Data"]
        elif "data" in data:
            d = data["data"]

        if not d:
            logger.warning(f"消息不含 data 字段: {data}")
            return

        if "CreateTime" in d:
            # 得到系统 UTF+8 的 ts
            tz_offset = datetime.timedelta(hours=8)
            tz = datetime.timezone(tz_offset)
            ts = datetime.datetime.now(tz).timestamp()
            create_time = d["CreateTime"]
            if create_time < ts - 30:
                logger.warning(f"消息时间戳过旧: {create_time}，当前时间戳: {ts}")
                return

        abm = AstrBotMessage()

        from_user_name = d["FromUserName"]["string"]  # 消息来源
        d["to_wxid"] = from_user_name  # 用于发信息

        abm.message_id = str(d.get("MsgId"))
        abm.session_id = from_user_name
        abm.self_id = data["Wxid"]  # 机器人的 wxid

        user_id = ""  # 发送人 wxid
        content = d["Content"]["string"]  # 消息内容

        at_me = False
        at_wxids = []
        if "@chatroom" in from_user_name:
            abm.type = MessageType.GROUP_MESSAGE
            _t = content.split(":\n")
            user_id = _t[0]
            content = _t[1]
            # at
            msg_source = d["MsgSource"]
            if "\u2005" in content:
                # at
                # content = content.split('\u2005')[1]
                content = re.sub(r"@[^\u2005]*\u2005", "", content)
                at_wxids = re.findall(
                    r"<atuserlist><!\[CDATA\[.*?(?:,|\b)([^,]+?)(?=,|\]\]></atuserlist>)",
                    msg_source,
                )

            abm.group_id = from_user_name

            if (
                f"<atuserlist><![CDATA[,{abm.self_id}]]>" in msg_source
                or f"<atuserlist><![CDATA[{abm.self_id}]]>" in msg_source
            ):
                at_me = True
            if "在群聊中@了你" in d.get("PushContent", ""):
                at_me = True
        else:
            abm.type = MessageType.FRIEND_MESSAGE
            user_id = from_user_name

        # 检查消息是否由自己发送，若是则忽略
        # 已经有可配置项专门配置是否需要响应自己的消息，因此这里注释掉。
        # if user_id == abm.self_id:
        #     logger.info("忽略自己发送的消息")
        #     return None

        abm.message = []

        # 解析用户真实名字
        user_real_name = "unknown"
        if abm.group_id:
            if (
                abm.group_id not in self.userrealnames
                or user_id not in self.userrealnames[abm.group_id]
            ):
                # 获取群成员列表，并且缓存
                if abm.group_id not in self.userrealnames:
                    self.userrealnames[abm.group_id] = {}
                member_list = await self.get_chatroom_member_list(abm.group_id)
                logger.debug(f"获取到 {abm.group_id} 的群成员列表。")
                if member_list and "memberList" in member_list:
                    for member in member_list["memberList"]:
                        self.userrealnames[abm.group_id][member["wxid"]] = member[
                            "nickName"
                        ]
                if user_id in self.userrealnames[abm.group_id]:
                    user_real_name = self.userrealnames[abm.group_id][user_id]
            else:
                user_real_name = self.userrealnames[abm.group_id][user_id]
        else:
            try:
                info = (await self.get_user_or_group_info(user_id))["data"][0]
                user_real_name = info["nickName"]
            except Exception as e:
                logger.debug(f"获取用户 {user_id} 昵称失败: {e}")
                user_real_name = user_id

        if at_me:
            abm.message.insert(0, At(qq=abm.self_id, name=self.nickname))
        for wxid in at_wxids:
            # 群聊里 At 其他人的列表
            _username = self.userrealnames.get(abm.group_id, {}).get(wxid, wxid)
            abm.message.append(At(qq=wxid, name=_username))

        abm.sender = MessageMember(user_id, user_real_name)
        abm.raw_message = d
        abm.message_str = ""

        if user_id == "weixin":
            # 忽略微信团队消息
            return

        # 不同消息类型
        match d["MsgType"]:
            case 1:
                # 文本消息
                abm.message.append(Plain(content))
                abm.message_str = content
            case 3:
                # 图片消息
                file_url = await self.multimedia_downloader.download_image(
                    self.appid, content
                )
                logger.debug(f"下载图片: {file_url}")
                file_path = await download_image_by_url(file_url)
                abm.message.append(Image(file=file_path, url=file_path))

            case 34:
                # 语音消息
                if "ImgBuf" in d and "buffer" in d["ImgBuf"]:
                    voice_data = base64.b64decode(d["ImgBuf"]["buffer"])
                    temp_dir = os.path.join(get_astrbot_data_path(), "temp")
                    file_path = os.path.join(
                        temp_dir, f"gewe_voice_{abm.message_id}.silk"
                    )

                    async with await anyio.open_file(file_path, "wb") as f:
                        await f.write(voice_data)
                    abm.message.append(Record(file=file_path, url=file_path))

            # 以下已知消息类型，没有业务处理，只是避免控制台打印太多的日志
            case 37:  # 好友申请
                logger.info("消息类型(37)：好友申请")
            case 42:  # 名片
                logger.info("消息类型(42)：名片")
            case 43:  # 视频
                video = Video(file="", cover=content)
                abm.message.append(video)
            case 47:  # emoji
                data_parser = GeweDataParser(content, abm.group_id == "")
                emoji = data_parser.parse_emoji()
                abm.message.append(emoji)
            case 48:  # 地理位置
                logger.info("消息类型(48)：地理位置")
            case 49:  # 公众号/文件/小程序/引用/转账/红包/视频号/群聊邀请
                data_parser = GeweDataParser(content, abm.group_id == "")
                segments = data_parser.parse_mutil_49()
                if segments:
                    abm.message.extend(segments)
                    for seg in segments:
                        if isinstance(seg, Plain):
                            abm.message_str += seg.text
            case 51:  # 帐号消息同步?
                logger.info("消息类型(51)：帐号消息同步？")
            case 10000:  # 被踢出群聊/更换群主/修改群名称
                logger.info("消息类型(10000)：被踢出群聊/更换群主/修改群名称")
            case 10002:  # 撤回/拍一拍/成员邀请/被移出群聊/解散群聊/群公告/群待办
                logger.info(
                    "消息类型(10002)：撤回/拍一拍/成员邀请/被移出群聊/解散群聊/群公告/群待办"
                )

            case _:
                logger.info(f"未实现的消息类型: {d['MsgType']}")
                abm.raw_message = d

        logger.debug(f"abm: {abm}")
        return abm

    async def _callback(self):
        data = await quart.request.json
        logger.debug(f"收到 gewechat 回调: {data}")

        if data.get("testMsg", None):
            return quart.jsonify({"r": "AstrBot ACK"})

        abm = None
        try:
            abm = await self._convert(data)
        except BaseException as e:
            logger.warning(
                f"尝试解析 GeweChat 下发的消息时遇到问题: {e}。下发消息内容: {data}。"
            )

        if abm:
            coro = getattr(self, "on_event_received")
            if coro:
                await coro(abm)

        return quart.jsonify({"r": "AstrBot ACK"})

    async def _register_file(self, file_path: str) -> str:
        """向 AstrBot 回调服务器 注册一个允许外部访问的文件。

        Args:
            file_path (str): 文件路径。
        Returns:
            str: 返回一个 auth_token，文件路径为 file_path。通过 /astrbot-gewechat/file/auth_token 得到文件。
        """
        async with self.lock:
            if not os.path.exists(file_path):
                raise Exception(f"文件不存在: {file_path}")

            file_token = str(uuid.uuid4())
            self.staged_files[file_token] = file_path
            return file_token

    async def _handle_file(self, file_token):
        async with self.lock:
            if file_token not in self.staged_files:
                logger.warning(f"请求的文件 {file_token} 不存在。")
                return quart.abort(404)
            if not os.path.exists(self.staged_files[file_token]):
                logger.warning(f"请求的文件 {self.staged_files[file_token]} 不存在。")
                return quart.abort(404)
            file_path = self.staged_files[file_token]
            self.staged_files.pop(file_token, None)
            return await quart.send_file(file_path)

    async def _set_callback_url(self):
        logger.info("设置回调，请等待...")
        await asyncio.sleep(3)
        async with aiohttp.ClientSession() as session:
            async with session.post(
                f"{self.base_url}/tools/setCallback",
                headers=self.headers,
                json={"token": self.token, "callbackUrl": self.callback_url},
            ) as resp:
                json_blob = await resp.json()
                logger.info(f"设置回调结果: {json_blob}")
                if json_blob["ret"] != 200:
                    raise Exception(f"设置回调失败: {json_blob}")
                logger.info(
                    f"将在 {self.callback_url} 上接收 gewechat 下发的消息。如果一直没收到消息请先尝试重启 AstrBot。如果仍没收到请到管理面板聊天页输入 /gewe_logout 重新登录。"
                )

    async def start_polling(self):
        threading.Thread(target=asyncio.run, args=(self._set_callback_url(),)).start()
        await self.server.run_task(
            host="0.0.0.0",
            port=self.port,
            shutdown_trigger=self.shutdown_trigger,
        )

    async def shutdown_trigger(self):
        await self.shutdown_event.wait()

    async def check_online(self, appid: str):
        """检查 APPID 对应的设备是否在线。"""
        async with aiohttp.ClientSession() as session:
            async with session.post(
                f"{self.base_url}/login/checkOnline",
                headers=self.headers,
                json={"appId": appid},
            ) as resp:
                json_blob = await resp.json()
                return json_blob["data"]

    async def logout(self):
        """登出 gewechat。"""
        if self.appid:
            online = await self.check_online(self.appid)
            if online:
                async with aiohttp.ClientSession() as session:
                    async with session.post(
                        f"{self.base_url}/login/logout",
                        headers=self.headers,
                        json={"appId": self.appid},
                    ) as resp:
                        json_blob = await resp.json()
                        logger.info(f"登出结果: {json_blob}")

    async def login(self):
        """登录 gewechat。一般来说插件用不到这个方法。"""
        if self.token is None:
            await self.get_token_id()

        self.multimedia_downloader = GeweDownloader(
            self.base_url, self.download_base_url, self.token
        )

        if self.appid:
            try:
                online = await self.check_online(self.appid)
                if online:
                    logger.info(f"APPID: {self.appid} 已在线")
                    return
            except Exception as e:
                logger.error(f"检查在线状态失败: {e}")
                sp.put(f"gewechat-appid-{self.nickname}", "")
                self.appid = None

        payload = {"appId": self.appid}

        if self.appid:
            logger.info(f"使用 APPID: {self.appid}, {self.nickname}")

        try:
            async with aiohttp.ClientSession() as session:
                async with session.post(
                    f"{self.base_url}/login/getLoginQrCode",
                    headers=self.headers,
                    json=payload,
                ) as resp:
                    json_blob = await resp.json()
                    if json_blob["ret"] != 200:
                        error_msg = json_blob.get("data", {}).get("msg", "")
                        if "设备不存在" in error_msg:
                            logger.error(
                                f"检测到无效的appid: {self.appid}，将清除并重新登录。"
                            )
                            sp.put(f"gewechat-appid-{self.nickname}", "")
                            self.appid = None
                            return await self.login()
                        else:
                            raise Exception(f"获取二维码失败: {json_blob}")
                    qr_data = json_blob["data"]["qrData"]
                    qr_uuid = json_blob["data"]["uuid"]
                    appid = json_blob["data"]["appId"]
                    logger.info(f"APPID: {appid}")
                    logger.warning(
                        f"请打开该网址，然后使用微信扫描二维码登录: https://api.cl2wm.cn/api/qrcode/code?text={qr_data}"
                    )
        except Exception as e:
            raise e

        # 执行登录
        retry_cnt = 64
        payload.update({"uuid": qr_uuid, "appId": appid})
        while retry_cnt > 0:
            retry_cnt -= 1

            # 需要验证码
            temp_dir = os.path.join(get_astrbot_data_path(), "temp")
            code_file_path = os.path.join(temp_dir, "gewe_code")
            if os.path.exists(code_file_path):
                with open(code_file_path, "r") as f:
                    code = f.read().strip()
                    if not code:
                        logger.warning(
                            "未找到验证码，请在管理面板聊天页输入 /gewe_code 验证码 来验证，如 /gewe_code 123456"
                        )
                        await asyncio.sleep(5)
                        continue
                    payload["captchCode"] = code
                    logger.info(f"使用验证码: {code}")
                    try:
                        os.remove(code_file_path)
                    except Exception:
                        logger.warning(f"删除验证码文件 {code_file_path} 失败。")

            async with aiohttp.ClientSession() as session:
                async with session.post(
                    f"{self.base_url}/login/checkLogin",
                    headers=self.headers,
                    json=payload,
                ) as resp:
                    json_blob = await resp.json()
                    logger.info(f"检查登录状态: {json_blob}")

                    ret = json_blob["ret"]
                    msg = ""
                    if json_blob["data"] and "msg" in json_blob["data"]:
                        msg = json_blob["data"]["msg"]
                    if ret == 500 and "安全验证码" in msg:
                        logger.warning(
                            "此次登录需要安全验证码，请在管理面板聊天页输入 /gewe_code 验证码 来验证，如 /gewe_code 123456"
                        )
                    else:
                        if "status" in json_blob["data"]:
                            status = json_blob["data"]["status"]
                            nickname = json_blob["data"].get("nickName", "")
                            if status == 1:
                                logger.info(f"等待确认...{nickname}")
                            elif status == 2:
                                logger.info(f"绿泡泡平台登录成功: {nickname}")
                                break
                            elif status == 0:
                                logger.info("等待扫码...")
                            else:
                                logger.warning(f"未知状态: {status}")
            await asyncio.sleep(5)

        if appid:
            sp.put(f"gewechat-appid-{self.nickname}", appid)
            self.appid = appid
            logger.info(f"已保存 APPID: {appid}")

    """API 部分。Gewechat 的 API 文档请参考: https://apifox.com/apidoc/shared/69ba62ca-cb7d-437e-85e4-6f3d3df271b1
    """

    async def get_chatroom_member_list(self, chatroom_wxid: str) -> dict:
        """获取群成员列表。

        Args:
            chatroom_wxid (str): 微信群聊的id。可以通过 event.get_group_id() 获取。

        Returns:
            dict: 返回群成员列表字典。其中键为 memberList 的值为群成员列表。
        """
        payload = {"appId": self.appid, "chatroomId": chatroom_wxid}

        async with aiohttp.ClientSession() as session:
            async with session.post(
                f"{self.base_url}/group/getChatroomMemberList",
                headers=self.headers,
                json=payload,
            ) as resp:
                json_blob = await resp.json()
                return json_blob["data"]

    async def post_text(self, to_wxid, content: str, ats: str = ""):
        """发送纯文本消息"""
        payload = {
            "appId": self.appid,
            "toWxid": to_wxid,
            "content": content,
        }
        if ats:
            payload["ats"] = ats

        async with aiohttp.ClientSession() as session:
            async with session.post(
                f"{self.base_url}/message/postText", headers=self.headers, json=payload
            ) as resp:
                json_blob = await resp.json()
                logger.debug(f"发送消息结果: {json_blob}")

    async def post_image(self, to_wxid, image_url: str):
        """发送图片消息"""
        payload = {
            "appId": self.appid,
            "toWxid": to_wxid,
            "imgUrl": image_url,
        }

        async with aiohttp.ClientSession() as session:
            async with session.post(
                f"{self.base_url}/message/postImage", headers=self.headers, json=payload
            ) as resp:
                json_blob = await resp.json()
                logger.debug(f"发送图片结果: {json_blob}")

    async def post_emoji(self, to_wxid, emoji_md5, emoji_size, cdnurl=""):
        """发送emoji消息"""
        payload = {
            "appId": self.appid,
            "toWxid": to_wxid,
            "emojiMd5": emoji_md5,
            "emojiSize": emoji_size,
        }

        # 优先表情包，若拿不到表情包的md5，就用当作图片发
        try:
            if emoji_md5 != "" and emoji_size != "":
                async with aiohttp.ClientSession() as session:
                    async with session.post(
                        f"{self.base_url}/message/postEmoji",
                        headers=self.headers,
                        json=payload,
                    ) as resp:
                        json_blob = await resp.json()
                        logger.info(
                            f"发送emoji消息结果: {json_blob.get('msg', '操作失败')}"
                        )
            else:
                await self.post_image(to_wxid, cdnurl)

        except Exception as e:
            logger.error(e)

    async def post_video(
        self, to_wxid, video_url: str, thumb_url: str, video_duration: int
    ):
        payload = {
            "appId": self.appid,
            "toWxid": to_wxid,
            "videoUrl": video_url,
            "thumbUrl": thumb_url,
            "videoDuration": video_duration,
        }
        async with aiohttp.ClientSession() as session:
            async with session.post(
                f"{self.base_url}/message/postVideo", headers=self.headers, json=payload
            ) as resp:
                json_blob = await resp.json()
                logger.debug(f"发送视频结果: {json_blob}")

    async def forward_video(self, to_wxid, cnd_xml: str):
        """转发视频

        Args:
            to_wxid (str): 发送给谁
            cnd_xml (str): 视频消息的cdn信息
        """
        payload = {
            "appId": self.appid,
            "toWxid": to_wxid,
            "xml": cnd_xml,
        }
        async with aiohttp.ClientSession() as session:
            async with session.post(
                f"{self.base_url}/message/forwardVideo",
                headers=self.headers,
                json=payload,
            ) as resp:
                json_blob = await resp.json()
                logger.debug(f"转发视频结果: {json_blob}")

    async def post_voice(self, to_wxid, voice_url: str, voice_duration: int):
        """发送语音信息

        Args:
            voice_url (str): 语音文件的网络链接
            voice_duration (int): 语音时长，毫秒
        """
        payload = {
            "appId": self.appid,
            "toWxid": to_wxid,
            "voiceUrl": voice_url,
            "voiceDuration": voice_duration,
        }

        logger.debug(f"发送语音: {payload}")

        async with aiohttp.ClientSession() as session:
            async with session.post(
                f"{self.base_url}/message/postVoice", headers=self.headers, json=payload
            ) as resp:
                json_blob = await resp.json()
                logger.info(f"发送语音结果: {json_blob.get('msg', '操作失败')}")

    async def post_file(self, to_wxid, file_url: str, file_name: str):
        """发送文件

        Args:
            to_wxid (string): 微信ID
            file_url (str): 文件的网络链接
            file_name (str): 文件名
        """
        payload = {
            "appId": self.appid,
            "toWxid": to_wxid,
            "fileUrl": file_url,
            "fileName": file_name,
        }

        async with aiohttp.ClientSession() as session:
            async with session.post(
                f"{self.base_url}/message/postFile", headers=self.headers, json=payload
            ) as resp:
                json_blob = await resp.json()
                logger.debug(f"发送文件结果: {json_blob}")

    async def add_friend(self, v3: str, v4: str, content: str):
        """申请添加好友"""
        payload = {
            "appId": self.appid,
            "scene": 3,
            "content": content,
            "v4": v4,
            "v3": v3,
            "option": 2,
        }

        async with aiohttp.ClientSession() as session:
            async with session.post(
                f"{self.base_url}/contacts/addContacts",
                headers=self.headers,
                json=payload,
            ) as resp:
                json_blob = await resp.json()
                logger.debug(f"申请添加好友结果: {json_blob}")
                return json_blob

    async def get_group(self, group_id: str):
        payload = {
            "appId": self.appid,
            "chatroomId": group_id,
        }

        async with aiohttp.ClientSession() as session:
            async with session.post(
                f"{self.base_url}/group/getChatroomInfo",
                headers=self.headers,
                json=payload,
            ) as resp:
                json_blob = await resp.json()
                logger.debug(f"获取群信息结果: {json_blob}")
                return json_blob

    async def get_group_member(self, group_id: str):
        payload = {
            "appId": self.appid,
            "chatroomId": group_id,
        }

        async with aiohttp.ClientSession() as session:
            async with session.post(
                f"{self.base_url}/group/getChatroomMemberList",
                headers=self.headers,
                json=payload,
            ) as resp:
                json_blob = await resp.json()
                logger.debug(f"获取群信息结果: {json_blob}")
                return json_blob

    async def accept_group_invite(self, url: str):
        """同意进群"""
        payload = {"appId": self.appid, "url": url}

        async with aiohttp.ClientSession() as session:
            async with session.post(
                f"{self.base_url}/group/agreeJoinRoom",
                headers=self.headers,
                json=payload,
            ) as resp:
                json_blob = await resp.json()
                logger.debug(f"获取群信息结果: {json_blob}")
                return json_blob

    async def add_group_member_to_friend(
        self, group_id: str, to_wxid: str, content: str
    ):
        payload = {
            "appId": self.appid,
            "chatroomId": group_id,
            "content": content,
            "memberWxid": to_wxid,
        }

        async with aiohttp.ClientSession() as session:
            async with session.post(
                f"{self.base_url}/group/addGroupMemberAsFriend",
                headers=self.headers,
                json=payload,
            ) as resp:
                json_blob = await resp.json()
                logger.debug(f"获取群信息结果: {json_blob}")
                return json_blob

    async def get_user_or_group_info(self, *ids):
        """
        获取用户或群组信息。

        :param ids: 可变数量的 wxid 参数
        """

        wxids_str = list(ids)

        payload = {
            "appId": self.appid,
            "wxids": wxids_str,  # 使用逗号分隔的字符串
        }

        async with aiohttp.ClientSession() as session:
            async with session.post(
                f"{self.base_url}/contacts/getDetailInfo",
                headers=self.headers,
                json=payload,
            ) as resp:
                json_blob = await resp.json()
                logger.debug(f"获取群信息结果: {json_blob}")
                return json_blob

    async def get_contacts_list(self):
        """
        获取通讯录列表
        见 https://apifox.com/apidoc/shared/69ba62ca-cb7d-437e-85e4-6f3d3df271b1/api-196794504
        """
        payload = {"appId": self.appid}

        async with aiohttp.ClientSession() as session:
            async with session.post(
                f"{self.base_url}/contacts/fetchContactsList",
                headers=self.headers,
                json=payload,
            ) as resp:
                json_blob = await resp.json()
                logger.debug(f"获取通讯录列表结果: {json_blob}")
                return json_blob
